"""Generated message classes for cloudcommerceconsumerprocurement version v1alpha1.

Enables consumers to procure products served by Cloud Marketplace platform
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'cloudcommerceconsumerprocurement'


class CloudcommerceconsumerprocurementBillingAccountsAccountsCreateRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsAccountsCreateRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1Account: A
      GoogleCloudCommerceConsumerProcurementV1alpha1Account resource to be
      passed as the request body.
    parent: Required. The parent resource of this account. This field is of
      the form "/". Currently supported type: 'billingAccounts/{billing-
      account-id}'
  """

  googleCloudCommerceConsumerProcurementV1alpha1Account = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Account', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsAccountsDeleteRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsAccountsDeleteRequest
  object.

  Fields:
    name: Required. The resource name of the account to delete.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementBillingAccountsAccountsGetRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsAccountsGetRequest
  object.

  Fields:
    name: Required. The resource name of the account to retrieve.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementBillingAccountsAccountsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsAccountsListRequest
  object.

  Fields:
    pageSize: The maximum number of entries that are requested. The default
      page size is 25 and the maximum page size is 200.
    pageToken: The token for fetching the next page.
    parent: Required. The parent resource to query for accounts. This field is
      of the form `billingAccounts/{billing-account-id}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudcommerceconsumerprocurementBillingAccountsConsentsCheckRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsConsentsCheckRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest
      resource to be passed as the request body.
    parent: Required. Parent of consents. Current supported format includes: -
      billingAccounts/{billing_account} - projects/{project_id}
  """

  googleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsConsentsGrantRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsConsentsGrantRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest
      resource to be passed as the request body.
    parent: Required. Parent of the consent to grant. Current supported format
      includes: - billingAccounts/{billing_account} - projects/{project_id}
  """

  googleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsConsentsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsConsentsListRequest
  object.

  Fields:
    agreement: Leave this field unset will return consents of agreements that
      user has access to. A valid format would be - commerceoffercatalog.googl
      eapis.com/billingAccounts/{billing_account}/offers/{offer_id}/agreements
      /{agreement_id}
    pageSize: The maximum number of results returned by this request.
    pageToken: The continuation token, which is used to page through large
      result sets. To get the next page of results, set this parameter to the
      value of `nextPageToken` from the previous response.
    parent: Required. Parent of consents. Current supported format includes: -
      billingAccounts/{billing_account} - projects/{project_id}
  """

  agreement = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class CloudcommerceconsumerprocurementBillingAccountsConsentsRevokeRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsConsentsRevokeRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest
      resource to be passed as the request body.
    name: Required. A consent to be reovked. Examples of valid names would be:
      - billingAccounts/{billing_account}/consents/{consent_id} -
      projects/{project_id}/consents/{consent_id}
  """

  googleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersCancelRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersCancelRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1CancelOrderRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1CancelOrderRequest
      resource to be passed as the request body.
    name: Required. The resource name of the order.
  """

  googleCloudCommerceConsumerProcurementV1alpha1CancelOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1CancelOrderRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersGetRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersGetRequest
  object.

  Fields:
    name: Required. The name of the order to retrieve.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersListMigratableRequest(_messages.Message):
  r"""A
  CloudcommerceconsumerprocurementBillingAccountsOrdersListMigratableRequest
  object.

  Fields:
    newParent: Required. The parent the Orders intend on migrating to. This
      field is of the form `billingAccounts/`. You must have the
      "consumerprocurement.orders.migrate" permission on new_parent also.
    pageSize: The maximum number of entries that are requested.
    pageToken: The token for fetching the next page.
    parent: Required. The parent resource to query for orders. This field is
      of the form `billingAccounts/`.
  """

  newParent = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersListRequest
  object.

  Fields:
    filter: Filter that you can use to limit the list request. A query string
      that can match a selected set of attributes with string values. For
      example, `display_name=abc`. Supported query attributes are *
      `display_name` If the query contains special characters other than
      letters, underscore, or digits, the phrase must be quoted with double
      quotes. For example, `display_name="foo:bar"`, where the display name
      needs to be quoted because it contains special character colon. Queries
      can be combined with `OR`, and `NOT` to form more complex queries. You
      can also group them to force a desired evaluation order. For example,
      `display_name=abc OR display_name=def`.
    pageSize: The maximum number of entries requested. The default page size
      is 25 and the maximum page size is 200.
    pageToken: The token for fetching the next page.
    parent: Required. The parent resource to query for orders. This field has
      the form `billingAccounts/{billing-account-id}`.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersMigrateRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersMigrateRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1MigrateOrderRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1MigrateOrderRequest
      resource to be passed as the request body.
    name: Required. Name of the order to migrate. This field is in this form:
      billingAccounts/{billing-account-id}/orders/{order-id}.
  """

  googleCloudCommerceConsumerProcurementV1alpha1MigrateOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1MigrateOrderRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersModifyRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersModifyRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequest
      resource to be passed as the request body.
    name: Required. Name of the order to update.
  """

  googleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOperationsGetRequest(_messages.Message):
  r"""A
  CloudcommerceconsumerprocurementBillingAccountsOrdersOperationsGetRequest
  object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsCreateRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsC
  reateRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1OrderAllocation: A
      GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation resource
      to be passed as the request body.
    parent: Required. The parent resource of this OrderAllocation. This field
      is of the form 'billingAccounts/{billing-account-id}/orders/{order-id}'.
  """

  googleCloudCommerceConsumerProcurementV1alpha1OrderAllocation = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsDeleteRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsD
  eleteRequest object.

  Fields:
    etag: The weak etag of the order allocation, which can be optionally set.
    name: Required. The resource name of the OrderAllocation to delete.
  """

  etag = _messages.StringField(1)
  name = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsGetRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsG
  etRequest object.

  Fields:
    name: Required. The name of the OrderAllocation to retrieve. This field is
      of the form `billingAccounts/{billing-account-id}/orders/{order-
      id}/orderAllocations/{order-allocation-id}`.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsL
  istRequest object.

  Fields:
    pageSize: The maximum number of entries that are requested. The default
      page size is 25 and the maximum page size is 200.
    pageToken: The token for fetching the next page.
    parent: Required. The parent resource to query for OrderAllocations. This
      field is of the form `billingAccounts/{billing-account-
      id}/orders/{order-id}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsPatchRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsP
  atchRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1OrderAllocation: A
      GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation resource
      to be passed as the request body.
    name: The resource name of the allocation. This field is of the form: `bil
      lingAccounts/{billing_account}/orders/{order}/orderAllocations/{order_al
      location}`.
    updateMask: Optional. The update mask that applies to the resource. See
      the [FieldMask definition] (https://developers.google.com/protocol-
      buffers/docs/reference/google.protobuf#fieldmask) for more details.
  """

  googleCloudCommerceConsumerProcurementV1alpha1OrderAllocation = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsReplaceRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAllocationsR
  eplaceRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocationsReque
      st: A GoogleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocati
      onsRequest resource to be passed as the request body.
    parent: Required. The parent resource to replace OrderAllocations for.
      This field is in this form: billingAccounts/{billing-account-
      id}/orders/{order-id}.
  """

  googleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocationsRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocationsRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAttributionsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAttributions
  ListRequest object.

  Fields:
    pageSize: The maximum number of entries returned per call.
    pageToken: The token for fetching the next page of entries.
    parent: Required. The parent Order to query for OrderAttributions. This
      field is of the form `billingAccounts/{billing-account-
      id}/orders/{order-id}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAttributionsPatchRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersOrderAttributions
  PatchRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1OrderAttribution: A
      GoogleCloudCommerceConsumerProcurementV1alpha1OrderAttribution resource
      to be passed as the request body.
    name: Output only. Resource name of the attribution configuration Format:
      billingAccounts/{billing_account}/orders/{order}/orderAttributions/{orde
      r_attribution} attribution_target references the Order parameter that
      defines the total attributable amount of this resource.
    updateMask: Optional. Mask used to indicate which parts of
      OrderAttribution are to be updated.
  """

  googleCloudCommerceConsumerProcurementV1alpha1OrderAttribution = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAttribution', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class CloudcommerceconsumerprocurementBillingAccountsOrdersPlaceRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementBillingAccountsOrdersPlaceRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1PlaceOrderRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1PlaceOrderRequest resource
      to be passed as the request body.
    parent: Required. The resource name of the parent resource. This field has
      the form `billingAccounts/{billing-account-id}`.
  """

  googleCloudCommerceConsumerProcurementV1alpha1PlaceOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1PlaceOrderRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementProjectsConsentsCheckRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsConsentsCheckRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest
      resource to be passed as the request body.
    parent: Required. Parent of consents. Current supported format includes: -
      billingAccounts/{billing_account} - projects/{project_id}
  """

  googleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementProjectsConsentsGrantRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsConsentsGrantRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest
      resource to be passed as the request body.
    parent: Required. Parent of the consent to grant. Current supported format
      includes: - billingAccounts/{billing_account} - projects/{project_id}
  """

  googleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementProjectsConsentsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsConsentsListRequest object.

  Fields:
    agreement: Leave this field unset will return consents of agreements that
      user has access to. A valid format would be - commerceoffercatalog.googl
      eapis.com/billingAccounts/{billing_account}/offers/{offer_id}/agreements
      /{agreement_id}
    pageSize: The maximum number of results returned by this request.
    pageToken: The continuation token, which is used to page through large
      result sets. To get the next page of results, set this parameter to the
      value of `nextPageToken` from the previous response.
    parent: Required. Parent of consents. Current supported format includes: -
      billingAccounts/{billing_account} - projects/{project_id}
  """

  agreement = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class CloudcommerceconsumerprocurementProjectsConsentsRevokeRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsConsentsRevokeRequest object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest: A
      GoogleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest
      resource to be passed as the request body.
    name: Required. A consent to be reovked. Examples of valid names would be:
      - billingAccounts/{billing_account}/consents/{consent_id} -
      projects/{project_id}/consents/{consent_id}
  """

  googleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementProjectsEntitlementsGetRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsEntitlementsGetRequest object.

  Fields:
    name: Required. The name of the entitlement to retrieve. This field is one
      of the following forms: `projects/{project-
      number}/entitlements/{entitlement-id}` `projects/{project-
      id}/entitlements/{entitlement-id}`.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementProjectsEntitlementsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsEntitlementsListRequest
  object.

  Fields:
    filter: Filter that can be used to limit the list request. A query string
      that can match a selected set of attributes with string values. For
      example `product_external_name=abc`. Supported query attributes are *
      `product_external_name` * `provider` If the query contains special
      characters other than letters, underscore, or digits, the phrase must be
      quoted with double quotes. For example,
      `product_external_name="foo:bar"`, where the product external name needs
      to be quoted because it contains special character colon. Queries can be
      combined with `OR`, and `NOT` to form more complex queries. You can also
      group them to force a desired evaluation order. For example,
      `product_external_name=abc OR product_external_name=def`.
    pageSize: The maximum number of entries requested. The default page size
      is 25 and the maximum page size is 200.
    pageToken: The token for fetching the next page.
    parent: Required. The parent resource to query for Entitlements. Currently
      the only parents supported are "projects/{project-number}" and
      "projects/{project-id}".
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class CloudcommerceconsumerprocurementProjectsFreeTrialsCreateRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsFreeTrialsCreateRequest
  object.

  Fields:
    googleCloudCommerceConsumerProcurementV1alpha1FreeTrial: A
      GoogleCloudCommerceConsumerProcurementV1alpha1FreeTrial resource to be
      passed as the request body.
    parent: Required. The parent resource to query for FreeTrials. Currently
      the only parent supported is "projects/{project-id}".
  """

  googleCloudCommerceConsumerProcurementV1alpha1FreeTrial = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1FreeTrial', 1)
  parent = _messages.StringField(2, required=True)


class CloudcommerceconsumerprocurementProjectsFreeTrialsGetRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsFreeTrialsGetRequest object.

  Fields:
    name: Required. The name of the freeTrial to retrieve. This field is of
      the form `projects/{project-id}/freeTrials/{freetrial-id}`.
  """

  name = _messages.StringField(1, required=True)


class CloudcommerceconsumerprocurementProjectsFreeTrialsListRequest(_messages.Message):
  r"""A CloudcommerceconsumerprocurementProjectsFreeTrialsListRequest object.

  Fields:
    filter: The filter that can be used to limit the list request. The filter
      is a query string that can match a selected set of attributes with
      string values. For example `product_external_name=1234-5678-ABCD-EFG`.
      Supported query attributes are * `product_external_name` * `provider` If
      the query contains special characters other than letters, underscore, or
      digits, the phrase must be quoted with double quotes. For example,
      `product_external_name="foo:bar"`, where the product name needs to be
      quoted because it contains special character colon. Queries can be
      combined with `AND`, `OR`, and `NOT` to form more complex queries. They
      can also be grouped to force a desired evaluation order. For example,
      `provider=providers/E-1234 OR provider=providers/5678 AND NOT
      (product_external_name=foo-product)`. Connective `AND` can be omitted
      between two predicates. For example `provider=providers/E-1234
      product_external_name=foo` is equivalent to `provider=providers/E-1234
      AND product_external_name=foo`.
    pageSize: The maximum number of entries that are requested. The default
      page size is 25 and the maximum page size is 200.
    pageToken: The token for fetching the next page.
    parent: Required. The parent resource to query for FreeTrials. Currently
      the only parent supported is "projects/{project-id}".
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1Account(_messages.Message):
  r"""Represents an account that was established by the customer with a
  service provider. When consuming services on external service provider's
  systems, the service provider generally needs to create a linked-account on
  their system to track customers. The account resource represents this
  relationship. Products/Services that are hosted by external service
  providers generally require an account to be present before they can be
  purchased and used. The metadata that indicates whether an Account is
  required for a purchase, or what parameters are needed for creating an
  Account is configured by service providers.

  Messages:
    PropertiesValue: Output only. Set of properties that the service provider
      supplied during account creation.

  Fields:
    approvals: Output only. The approvals for this account. These approvals
      are used to track actions that are permitted or have been completed by a
      customer within the context of the provider. This might include a sign
      up flow or a provisioning step, for example, that the provider can admit
      to having happened.
    createTime: Output only. The creation timestamp.
    name: Output only. The resource name of the account. Account names have
      the form `billingAccounts/{billing_account_id}/accounts/{account_id}`.
    properties: Output only. Set of properties that the service provider
      supplied during account creation.
    provider: Required. The identifier of the service provider that this
      account was created against. Provider has the format of
      `providers/{provider_id}`.
    updateTime: Output only. The last update timestamp.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PropertiesValue(_messages.Message):
    r"""Output only. Set of properties that the service provider supplied
    during account creation.

    Messages:
      AdditionalProperty: An additional property for a PropertiesValue object.

    Fields:
      additionalProperties: Additional properties of type PropertiesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PropertiesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  approvals = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1AccountApproval', 1, repeated=True)
  createTime = _messages.StringField(2)
  name = _messages.StringField(3)
  properties = _messages.MessageField('PropertiesValue', 4)
  provider = _messages.StringField(5)
  updateTime = _messages.StringField(6)


class GoogleCloudCommerceConsumerProcurementV1alpha1AccountApproval(_messages.Message):
  r"""An approval for some action on an account.

  Enums:
    StateValueValuesEnum: The state of the approval.

  Fields:
    name: The name of the approval.
    reason: An explanation for the state of the approval.
    state: The state of the approval.
    updateTime: The last update timestamp of the approval.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""The state of the approval.

    Values:
      STATE_UNSPECIFIED: Sentinel value; do not use.
      PENDING: The approval is pending response from the provider. The
        approval state can transition to Account.Approval.State.APPROVED or
        Account.Approval.State.REJECTED.
      APPROVED: The approval has been granted by the provider.
      REJECTED: The approval has been rejected by the provider. A provider may
        choose to approve a previously rejected approval, so is it possible to
        transition to Account.Approval.State.APPROVED.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    APPROVED = 2
    REJECTED = 3

  name = _messages.StringField(1)
  reason = _messages.StringField(2)
  state = _messages.EnumField('StateValueValuesEnum', 3)
  updateTime = _messages.StringField(4)


class GoogleCloudCommerceConsumerProcurementV1alpha1AddOnDetails(_messages.Message):
  r"""Information about an add-on line item.

  Fields:
    isAddOn: Output only. Indicates whether this item is an add-on.
  """

  isAddOn = _messages.BooleanField(1)


class GoogleCloudCommerceConsumerProcurementV1alpha1CancelOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CancelOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1CancelOrderRequest(_messages.Message):
  r"""Request message for ConsumerProcurementService.CancelOrder.

  Enums:
    CancellationPolicyValueValuesEnum: Cancellation policy of this request.

  Fields:
    cancellationPolicy: Cancellation policy of this request.
    etag: The weak etag, which can be optionally populated, of the order that
      this cancel request is based on. Validation checking will only happen if
      the invoker supplies this field.
  """

  class CancellationPolicyValueValuesEnum(_messages.Enum):
    r"""Cancellation policy of this request.

    Values:
      CANCELLATION_POLICY_UNSPECIFIED: If unspecified, cancellation will try
        to cancel the order, if order cannot be immediately cancelled, auto
        renewal will be turned off. However, caller should avoid using the
        value as it will yield a non-deterministic result. This is still
        supported mainly to maintain existing integrated usages and ensure
        backwards compatibility.
      CANCELLATION_POLICY_CANCEL_IMMEDIATELY: Request will cancel the whole
        order immediately, if order cannot be immediately cancelled, the
        request will fail.
      CANCELLATION_POLICY_CANCEL_AT_TERM_END: Request will cancel the auto
        renewal, if order is not subscription based, the request will fail.
    """
    CANCELLATION_POLICY_UNSPECIFIED = 0
    CANCELLATION_POLICY_CANCEL_IMMEDIATELY = 1
    CANCELLATION_POLICY_CANCEL_AT_TERM_END = 2

  cancellationPolicy = _messages.EnumField('CancellationPolicyValueValuesEnum', 1)
  etag = _messages.StringField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1CheckConsentRequest(_messages.Message):
  r"""Request message for check consents.

  Fields:
    agreement: Required. Agreement to be checked against. A valid format would
      be - commerceoffercatalog.googleapis.com/billingAccounts/{billing_accoun
      t}/offers/{offer_id}/agreements/{agreement_id}
    financialContract: Financial contract this consent applies to. This is a
      system full resource name. E.g.: //commerceoffercatalog.googleapis.com/b
      illingAccounts/{billing_account}/offers/{offer-id}
    languageCode: The language code is used to find the agreement document if
      check consent doesn't pass. If this field is set, the method will
      attempt to locate the agreement document written in that language. If
      such document cannot be found, the request will fail. If this field is
      not set, the method will try two strategies in the following order: 1)
      reuse the language of document associated with the most recent consent,
      2) use the document written in the default language, while default
      language is set by the agreement owner. If neither strategy works, the
      request will fail. Please follow BCP 47
      (https://www.w3.org/International/articles/bcp47/) for the language
      string.
    offer: Offer associated with the consent. Formats include "commerceofferca
      talog.googleapis.com/billingAccounts/{billing_account}/offers/{offer_id}
      ". "commerceoffercatalog.googleapis.com/services/{service}/offers/{offer
      _id}".
  """

  agreement = _messages.StringField(1)
  financialContract = _messages.StringField(2)
  languageCode = _messages.StringField(3)
  offer = _messages.StringField(4)


class GoogleCloudCommerceConsumerProcurementV1alpha1CheckConsentResponse(_messages.Message):
  r"""Response for check consent.

  Fields:
    consent: The Consent for this agreement if a consent is active.
  """

  consent = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Consent', 1)


class GoogleCloudCommerceConsumerProcurementV1alpha1Consent(_messages.Message):
  r"""A consent resource represents the relationship between a user and an
  agreement.

  Enums:
    StateValueValuesEnum: Output only. State of current consent.

  Fields:
    agreementDocument: Full name of the agreement document that was agreed to
      for this consent, ## in the format of one of: commerceoffercatalog.googl
      eapis.com/billingAccounts/{billing_account}/offers/{offer_id}/agreements
      /{agreement_id}/documents/{document_id}
    createTime: Output only. The creation time of current consent.
    financialContract: Financial contracts linked to this consent.
    name: The resource name of a consent. An examples of valid names would be
      in the format of: -
      "billingAccounts/{billing_account}/consents/{consent}". -
      "projects/{project_number}/consents/{consent}".
    offer: The name of the offer linked to this consent. It is in the format
      of: "commerceoffercatalog.googleapis.com/billingAccounts/{billing_accoun
      t}/offers/{offer_id}". "commerceoffercatalog.googleapis.com/services/{se
      rvice}/offers/{offer_id}".
    state: Output only. State of current consent.
    updateTime: Output only. The update time of current consent.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of current consent.

    Values:
      STATE_UNSPECIFIED: Unspecified value for the state. Sentinel value; do
        not use.
      ACTIVE: Represent the approved state of the consent.
      REVOKED: Represent the revoked state of the consent.
    """
    STATE_UNSPECIFIED = 0
    ACTIVE = 1
    REVOKED = 2

  agreementDocument = _messages.StringField(1)
  createTime = _messages.StringField(2)
  financialContract = _messages.StringField(3)
  name = _messages.StringField(4)
  offer = _messages.StringField(5)
  state = _messages.EnumField('StateValueValuesEnum', 6)
  updateTime = _messages.StringField(7)


class GoogleCloudCommerceConsumerProcurementV1alpha1CreateAccountMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CreateAccount.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1CreateFreeTrialMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CreateFreeTrial.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1CreateOrderAllocationMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CreateOrderAllocation.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1CustomPricing(_messages.Message):
  r"""Information about custom pricing on a resource.

  Fields:
    endTime: The end time of the custom pricing.
  """

  endTime = _messages.StringField(1)


class GoogleCloudCommerceConsumerProcurementV1alpha1DeleteAccountMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.DeleteAccount.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1DeleteOrderAllocationMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.DeleteOrderAllocation.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1Entitlement(_messages.Message):
  r"""Entitlement represents the ability to use a product or services
  associated with a purchase within a Project. When the customer creates an
  Order, the system will create Entitlement resources under projects
  associated with the same billing account as the order, for all
  products/services procured in the order. Users can enable/disable
  Entitlements to allow/disallow using the product/service in a project. Next
  Id: 22

  Enums:
    StateValueValuesEnum: Output only. The state of the entitlement.

  Fields:
    changeHistory: Output only. Changes that are not pending anymore, e.g. it
      was effective at some point, or the change was reverted by the customer,
      or the change was rejected by partner. No more operations are allowed on
      these changes.
    createTime: Output only. The create timestamp.
    flavorExternalName: Output only. External name of the flavor this
      entitlement is created against. This field is populated when entitlement
      has currently associated flavor, it is empty when entitlement is
      UNAVAILABLE (if order is pending activation or order is already
      cancelled).
    name: Output only. The resource Name of the Entitlement. Entitlement names
      have the form `projects/{project_id}/entitlements/{entitlement_id}`.
    order: Output only. Order associated with this Entitlement.
    pendingChange: Output only. A change which is pending and not yet
      effective.
    productExternalName: Output only. External name of the product this
      entitlement is created against.
    provider: Output only. Provider associated with this Entitlement. In the
      format of `providers/{provider_id}`.
    services: Output only. Services associated with the entitlement.
    state: Output only. The state of the entitlement.
    stateReason: Output only. An explanation for the entitlement's state.
      Mainly used in the case of
      `EntitlementState.ENTITLEMENT_STATE_UNAVAILABLE` states to explain why
      the entitlement is unavailable.
    updateTime: Output only. The last update timestamp.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the entitlement.

    Values:
      ENTITLEMENT_STATE_UNSPECIFIED: Sentinel value. Do not use.
      ENTITLEMENT_STATE_UNAVAILABLE: Indicates that the entitlement is
        unavailable and cannot be enabled.
      ENTITLEMENT_STATE_ENABLED: Indicates that the entitlement is enabled.
        The procured item is now usable.
      ENTITLEMENT_STATE_DISABLED: Indicates that the entitlement is disabled.
        The procured item is not usable.
    """
    ENTITLEMENT_STATE_UNSPECIFIED = 0
    ENTITLEMENT_STATE_UNAVAILABLE = 1
    ENTITLEMENT_STATE_ENABLED = 2
    ENTITLEMENT_STATE_DISABLED = 3

  changeHistory = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementChange', 1, repeated=True)
  createTime = _messages.StringField(2)
  flavorExternalName = _messages.StringField(3)
  name = _messages.StringField(4)
  order = _messages.StringField(5)
  pendingChange = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementChange', 6)
  productExternalName = _messages.StringField(7)
  provider = _messages.StringField(8)
  services = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementService', 9, repeated=True)
  state = _messages.EnumField('StateValueValuesEnum', 10)
  stateReason = _messages.StringField(11)
  updateTime = _messages.StringField(12)


class GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementChange(_messages.Message):
  r"""Entitlement change information. Next Id: 8

  Enums:
    ChangeStateValueValuesEnum: Output only. State of the change.
    ChangeStateReasonTypeValueValuesEnum: Output only. Predefined enum types
      for why this change is in current state.

  Fields:
    changeEffectiveTime: Output only. A time at which the change became or
      will become (in case of pending change) effective.
    changeState: Output only. State of the change.
    changeStateReasonType: Output only. Predefined enum types for why this
      change is in current state.
    newFlavorExternalName: Output only. Flavor external name after the change.
    oldFlavorExternalName: Output only. Flavor external name before the
      change.
  """

  class ChangeStateReasonTypeValueValuesEnum(_messages.Enum):
    r"""Output only. Predefined enum types for why this change is in current
    state.

    Values:
      CHANGE_STATE_REASON_TYPE_UNSPECIFIED: Default value, indicating there is
        no predefined type for change state reason.
      CHANGE_STATE_REASON_TYPE_EXPIRED: Change is in current state due to term
        expiration.
      CHANGE_STATE_REASON_TYPE_USER_CANCELLED: Change is in current state due
        to user explicit cancellation.
      CHANGE_STATE_REASON_TYPE_SYSTEM_CANCELLED: Change is in current state
        due to system cancellation.
    """
    CHANGE_STATE_REASON_TYPE_UNSPECIFIED = 0
    CHANGE_STATE_REASON_TYPE_EXPIRED = 1
    CHANGE_STATE_REASON_TYPE_USER_CANCELLED = 2
    CHANGE_STATE_REASON_TYPE_SYSTEM_CANCELLED = 3

  class ChangeStateValueValuesEnum(_messages.Enum):
    r"""Output only. State of the change.

    Values:
      CHANGE_STATE_UNSPECIFIED: Sentinel value. Do not use.
      CHANGE_STATE_PENDING_APPROVAL: Change is in this state when a change is
        initiated and waiting for partner approval.
      CHANGE_STATE_APPROVED: Change is in this state, if the change was
        approved by partner or auto-approved but is pending to be effective.
        The change can be overwritten or cancelled depending on the new line
        item info property (pending Private Offer change cannot be cancelled
        and can only be overwritten by another Private Offer).
      CHANGE_STATE_COMPLETED: Change is in this state, if the change was
        activated and completed successfully.
      CHANGE_STATE_REJECTED: Change is in this state, if the change was
        rejected by partner.
      CHANGE_STATE_ABANDONED: Change is in this state, if it was abandoned by
        user.
      CHANGE_STATE_ACTIVATING: Change is in this state, if it is going through
        downstream provision, the change cannot be overwritten or cancelled in
        this state.
    """
    CHANGE_STATE_UNSPECIFIED = 0
    CHANGE_STATE_PENDING_APPROVAL = 1
    CHANGE_STATE_APPROVED = 2
    CHANGE_STATE_COMPLETED = 3
    CHANGE_STATE_REJECTED = 4
    CHANGE_STATE_ABANDONED = 5
    CHANGE_STATE_ACTIVATING = 6

  changeEffectiveTime = _messages.StringField(1)
  changeState = _messages.EnumField('ChangeStateValueValuesEnum', 2)
  changeStateReasonType = _messages.EnumField('ChangeStateReasonTypeValueValuesEnum', 3)
  newFlavorExternalName = _messages.StringField(4)
  oldFlavorExternalName = _messages.StringField(5)


class GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementInfo(_messages.Message):
  r"""The benefit associated with a product purchase.

  Fields:
    id: ID of the entitlement info. Unique across all orders.
    services: The names of the Google Service Infrastructure services to
      enable.
  """

  id = _messages.StringField(1)
  services = _messages.StringField(2, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementService(_messages.Message):
  r"""A service associated with the entitlement.

  Enums:
    ServiceStateValueValuesEnum: Current state of the service within project.

  Fields:
    serviceName: The name of the service.
    serviceState: Current state of the service within project.
  """

  class ServiceStateValueValuesEnum(_messages.Enum):
    r"""Current state of the service within project.

    Values:
      STATE_UNSPECIFIED: Sentinel value. Do not use.
      ENABLED: Indicates service is enabled within project.
      DISABLED: Indicates service is disabled within project.
    """
    STATE_UNSPECIFIED = 0
    ENABLED = 1
    DISABLED = 2

  serviceName = _messages.StringField(1)
  serviceState = _messages.EnumField('ServiceStateValueValuesEnum', 2)


class GoogleCloudCommerceConsumerProcurementV1alpha1FreeTrial(_messages.Message):
  r"""FreeTrial represents the free trial created for a specific product and
  billing account with argentum. Free Trial resources are created by placing
  orders for 3p non-VM products, or just enabling free trials for 1p products
  and 3P VM products. Next Id: 6

  Fields:
    credit: Output only. Credit tracking the real time credit status.
    name: Output only. The resource name of the free trial item. This field is
      of the form: `projects/{project}/freeTrials/{free_trial}`. Present if
      free trial is created under the project's associated billing account for
      3p, or free trial is enabled for 1p product.
    productExternalName: External name for the product for which free trial
      exist.
    provider: Provider of the products for which free trial exist. Provider
      has the format of `providers/{provider_id}`.
  """

  credit = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1FreeTrialCredit', 1)
  name = _messages.StringField(2)
  productExternalName = _messages.StringField(3)
  provider = _messages.StringField(4)


class GoogleCloudCommerceConsumerProcurementV1alpha1FreeTrialCredit(_messages.Message):
  r"""Credit represents the real time credit information.

  Fields:
    creationDate: Date credit was created.
    endTime: When the credit expires. If empty then there's no upper bound of
      credit's effective timespan (i.e. the credit never expires).
    remainingAmount: The amount of the credit remaining.
    startTime: When the credit becomes effective. If empty then there's no
      lower bound of credit's effective timespan (i.e. the credit becomes
      effective at the time of its creation). For credit creation, this cannot
      be in the past.
    value: The value of the credit.
  """

  creationDate = _messages.StringField(1)
  endTime = _messages.StringField(2)
  remainingAmount = _messages.MessageField('GoogleTypeMoney', 3)
  startTime = _messages.StringField(4)
  value = _messages.MessageField('GoogleTypeMoney', 5)


class GoogleCloudCommerceConsumerProcurementV1alpha1GrantConsentRequest(_messages.Message):
  r"""Request message to grant consent.

  Fields:
    consent: Required. A consent to be granted. Set only agreement_document
      field.
    validateOnly: Optional. Flag is used to dry run grant consent behavior for
      the VMs and K8s products. If set, returns empty consent if consent can
      be granted. If unset, grants consent if consent can be granted.
  """

  consent = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Consent', 1)
  validateOnly = _messages.BooleanField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1LineItem(_messages.Message):
  r"""A single item within an order.

  Fields:
    changeHistory: Output only. Changes made on the item that are not pending
      anymore which might be because they already took effect, were reverted
      by the customer, or were rejected by the partner. No more operations are
      allowed on these changes.
    lineItemId: Output only. Line item ID.
    lineItemInfo: Output only. Current state and information of this item. It
      tells what, e.g. which offer, is currently effective.
    pendingChange: Output only. A change made on the item which is pending and
      not yet effective. Absence of this field indicates the line item is not
      undergoing a change.
  """

  changeHistory = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemChange', 1, repeated=True)
  lineItemId = _messages.StringField(2)
  lineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 3)
  pendingChange = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemChange', 4)


class GoogleCloudCommerceConsumerProcurementV1alpha1LineItemChange(_messages.Message):
  r"""A change made on a line item.

  Enums:
    ChangeStateValueValuesEnum: Output only. State of the change.
    ChangeStateReasonTypeValueValuesEnum: Output only. Predefined enum types
      for why this line item change is in current state. For example, a line
      item change's state could be `LINE_ITEM_CHANGE_STATE_COMPLETED` because
      of end-of-term expiration, immediate cancellation initiated by the user,
      or system-initiated cancellation.
    ChangeTypeValueValuesEnum: Required. Type of the change to make.

  Fields:
    changeEffectiveTime: Output only. A time at which the change became or
      will become (in case of pending change) effective.
    changeId: Output only. Change ID. All changes made within one order update
      operation have the same change_id.
    changeState: Output only. State of the change.
    changeStateReasonType: Output only. Predefined enum types for why this
      line item change is in current state. For example, a line item change's
      state could be `LINE_ITEM_CHANGE_STATE_COMPLETED` because of end-of-term
      expiration, immediate cancellation initiated by the user, or system-
      initiated cancellation.
    changeType: Required. Type of the change to make.
    createTime: Output only. The time when change was initiated.
    newLineItemInfo: Line item info after the change.
    oldLineItemInfo: Output only. Line item info before the change.
    stateReason: Output only. Provider-supplied message explaining the
      LineItemChange's state. Mainly used to communicate progress and ETA for
      provisioning in the case of `PENDING_APPROVAL`, and to explain why the
      change request was denied or canceled in the case of `REJECTED` and
      `CANCELED` states.
    updateTime: Output only. The time when change was updated, e.g.
      approved/rejected by partners or cancelled by the user.
  """

  class ChangeStateReasonTypeValueValuesEnum(_messages.Enum):
    r"""Output only. Predefined enum types for why this line item change is in
    current state. For example, a line item change's state could be
    `LINE_ITEM_CHANGE_STATE_COMPLETED` because of end-of-term expiration,
    immediate cancellation initiated by the user, or system-initiated
    cancellation.

    Values:
      LINE_ITEM_CHANGE_STATE_REASON_TYPE_UNSPECIFIED: Default value,
        indicating there's no predefined type for change state reason.
      LINE_ITEM_CHANGE_STATE_REASON_TYPE_EXPIRED: Change is in current state
        due to term expiration.
      LINE_ITEM_CHANGE_STATE_REASON_TYPE_USER_CANCELLED: Change is in current
        state due to user-initiated cancellation.
      LINE_ITEM_CHANGE_STATE_REASON_TYPE_SYSTEM_CANCELLED: Change is in
        current state due to system-initiated cancellation.
    """
    LINE_ITEM_CHANGE_STATE_REASON_TYPE_UNSPECIFIED = 0
    LINE_ITEM_CHANGE_STATE_REASON_TYPE_EXPIRED = 1
    LINE_ITEM_CHANGE_STATE_REASON_TYPE_USER_CANCELLED = 2
    LINE_ITEM_CHANGE_STATE_REASON_TYPE_SYSTEM_CANCELLED = 3

  class ChangeStateValueValuesEnum(_messages.Enum):
    r"""Output only. State of the change.

    Values:
      LINE_ITEM_CHANGE_STATE_UNSPECIFIED: Sentinel value. Do not use.
      LINE_ITEM_CHANGE_STATE_PENDING_APPROVAL: Change is in this state when a
        change is initiated and waiting for partner approval.
      LINE_ITEM_CHANGE_STATE_APPROVED: Change is in this state after it's
        approved by the partner or auto-approved but before it takes effect.
        The change can be overwritten or cancelled depending on the new line
        item info property (pending Private Offer change cannot be cancelled
        and can only be overwritten by another Private Offer).
      LINE_ITEM_CHANGE_STATE_COMPLETED: Change is in this state after it's
        been activated.
      LINE_ITEM_CHANGE_STATE_REJECTED: Change is in this state if it was
        rejected by the partner.
      LINE_ITEM_CHANGE_STATE_ABANDONED: Change is in this state if it was
        abandoned by the user.
      LINE_ITEM_CHANGE_STATE_ACTIVATING: Change is in this state if it's
        currently being provisioned downstream. The change can't be
        overwritten or cancelled when it's in this state.
    """
    LINE_ITEM_CHANGE_STATE_UNSPECIFIED = 0
    LINE_ITEM_CHANGE_STATE_PENDING_APPROVAL = 1
    LINE_ITEM_CHANGE_STATE_APPROVED = 2
    LINE_ITEM_CHANGE_STATE_COMPLETED = 3
    LINE_ITEM_CHANGE_STATE_REJECTED = 4
    LINE_ITEM_CHANGE_STATE_ABANDONED = 5
    LINE_ITEM_CHANGE_STATE_ACTIVATING = 6

  class ChangeTypeValueValuesEnum(_messages.Enum):
    r"""Required. Type of the change to make.

    Values:
      LINE_ITEM_CHANGE_TYPE_UNSPECIFIED: Sentinel value. Do not use.
      LINE_ITEM_CHANGE_TYPE_CREATE: The change is to create a new line item.
      LINE_ITEM_CHANGE_TYPE_UPDATE: The change is to update an existing line
        item.
      LINE_ITEM_CHANGE_TYPE_CANCEL: The change is to cancel an existing line
        item.
      LINE_ITEM_CHANGE_TYPE_REVERT_CANCELLATION: The change is to revert a
        cancellation.
    """
    LINE_ITEM_CHANGE_TYPE_UNSPECIFIED = 0
    LINE_ITEM_CHANGE_TYPE_CREATE = 1
    LINE_ITEM_CHANGE_TYPE_UPDATE = 2
    LINE_ITEM_CHANGE_TYPE_CANCEL = 3
    LINE_ITEM_CHANGE_TYPE_REVERT_CANCELLATION = 4

  changeEffectiveTime = _messages.StringField(1)
  changeId = _messages.StringField(2)
  changeState = _messages.EnumField('ChangeStateValueValuesEnum', 3)
  changeStateReasonType = _messages.EnumField('ChangeStateReasonTypeValueValuesEnum', 4)
  changeType = _messages.EnumField('ChangeTypeValueValuesEnum', 5)
  createTime = _messages.StringField(6)
  newLineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 7)
  oldLineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 8)
  stateReason = _messages.StringField(9)
  updateTime = _messages.StringField(10)


class GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo(_messages.Message):
  r"""Line item information.

  Messages:
    SystemPropertiesValue: Output only. System provided key value pairs.

  Fields:
    addOnDetails: Output only. Add on information of a line item, if
      applicable.
    customPricing: Output only. The custom pricing information for this line
      item, if applicable.
    entitlementInfo: Output only. Entitlement info associated with this line
      item.
    flavorExternalName: External name of the flavor being purchased.
    offer: Optional. The name of the offer can have either of these formats:
      'billingAccounts/{billing_account}/offers/{offer}', or
      'services/{service}/standardOffers/{offer}'.
    parameters: Optional. User-provided parameters.
    productExternalName: External name of the product being purchased.
    quoteExternalName: Output only. External name of the quote this product is
      associated with. Present if the product is part of a Quote.
    subscription: Output only. Information about the subscription created, if
      applicable.
    systemProperties: Output only. System provided key value pairs.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class SystemPropertiesValue(_messages.Message):
    r"""Output only. System provided key value pairs.

    Messages:
      AdditionalProperty: An additional property for a SystemPropertiesValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        SystemPropertiesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a SystemPropertiesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  addOnDetails = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1AddOnDetails', 1)
  customPricing = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1CustomPricing', 2)
  entitlementInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1EntitlementInfo', 3)
  flavorExternalName = _messages.StringField(4)
  offer = _messages.StringField(5)
  parameters = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Parameter', 6, repeated=True)
  productExternalName = _messages.StringField(7)
  quoteExternalName = _messages.StringField(8)
  subscription = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Subscription', 9)
  systemProperties = _messages.MessageField('SystemPropertiesValue', 10)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListAccountsResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListAccounts.

  Fields:
    accounts: The list of accounts in this response.
    nextPageToken: The token for fetching the next page.
  """

  accounts = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Account', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListConsentsResponse(_messages.Message):
  r"""Response message for the list consent request.

  Fields:
    consents: Consents matching the request.
    nextPageToken: Pagination token for large results.
  """

  consents = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Consent', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListEntitlementsResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListEntitlements.

  Fields:
    entitlements: The list of Entitlements in this response.
    nextPageToken: The token for fetching the next page.
  """

  entitlements = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Entitlement', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListFreeTrialsResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListFreeTrials.

  Fields:
    freeTrials: The list of FreeTrialss in this response.
    nextPageToken: The token for fetching the next page.
  """

  freeTrials = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1FreeTrial', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListMigratableOrdersResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListMigratableOrders.

  Fields:
    migratableOrders: The list of orders in this response.
    nextPageToken: The token for fetching the next page.
  """

  migratableOrders = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Order', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListOrderAllocationsResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListOrderAllocations.

  Fields:
    nextPageToken: The token for fetching the next page.
    orderAllocations: The list of OrderAllocations in this response.
  """

  nextPageToken = _messages.StringField(1)
  orderAllocations = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation', 2, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListOrderAttributionsResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListOrderAttributions.

  Fields:
    nextPageToken: The token for fetching the next page of entries.
    orderAttributions: The OrderAttributions from this response
  """

  nextPageToken = _messages.StringField(1)
  orderAttributions = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAttribution', 2, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1ListOrdersResponse(_messages.Message):
  r"""Response message for ConsumerProcurementService.ListOrders.

  Fields:
    nextPageToken: The token for fetching the next page.
    orders: The list of orders in this response.
  """

  nextPageToken = _messages.StringField(1)
  orders = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1Order', 2, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1MigrateOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.MigrateOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1MigrateOrderRequest(_messages.Message):
  r"""Request message for ConsumerProcurementService.MigrateOrder.

  Fields:
    newParent: Required. Destination billing account where the order will be
      migrated to. This field is of the form: 'billingAccounts/{billing-
      account-id}.
  """

  newParent = _messages.StringField(1)


class GoogleCloudCommerceConsumerProcurementV1alpha1ModifyOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.ModifyOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequest(_messages.Message):
  r"""Request message for ConsumerProcurementService.ModifyOrder. Next Id: 10

  Fields:
    displayName: Optional. Updated display name of the order, leave as empty
      if you do not want to update current display name.
    etag: The weak etag, which can be optionally populated, of the order that
      this modify request is based on. Validation checking will only happen if
      the invoker supplies this field.
    modifications: Optional. Modifications for an existing Order created by an
      Offer. Required when Offer based Order is being modified, except for
      when going from an offer to a public plan.
    modifyProductsOrderRequest: Required. Modifies an existing non-quote
      order. Should only be used for offer-based orders when going from an
      offer to a public plan.
    modifyQuoteOrderRequest: Required. Modifies an existing order for quote.
  """

  displayName = _messages.StringField(1)
  etag = _messages.StringField(2)
  modifications = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequestModification', 3, repeated=True)
  modifyProductsOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ModifyProductsOrderRequest', 4)
  modifyQuoteOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ModifyQuoteOrderRequest', 5)


class GoogleCloudCommerceConsumerProcurementV1alpha1ModifyOrderRequestModification(_messages.Message):
  r"""Modifications to make on the order.

  Enums:
    AutoRenewalBehaviorValueValuesEnum: Auto renewal behavior of the
      subscription for the update. Applied when change_type is
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE]. Follows plan default
      config when this field is not specified.
    ChangeTypeValueValuesEnum: Required. Type of change to make.

  Fields:
    autoRenewalBehavior: Auto renewal behavior of the subscription for the
      update. Applied when change_type is
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE]. Follows plan default
      config when this field is not specified.
    changeType: Required. Type of change to make.
    lineItemId: ID of the existing line item to make change to. Required when
      change type is [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE] or
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_CANCEL].
    newLineItemInfo: The line item to update to. Required when change_type is
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_CREATE] or
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE].
  """

  class AutoRenewalBehaviorValueValuesEnum(_messages.Enum):
    r"""Auto renewal behavior of the subscription for the update. Applied when
    change_type is [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE]. Follows
    plan default config when this field is not specified.

    Values:
      AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED: If unspecified, the auto renewal
        behavior will follow the default config.
      AUTO_RENEWAL_BEHAVIOR_ENABLE: Auto Renewal will be enabled on
        subscription.
      AUTO_RENEWAL_BEHAVIOR_DISABLE: Auto Renewal will be disabled on
        subscription.
    """
    AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED = 0
    AUTO_RENEWAL_BEHAVIOR_ENABLE = 1
    AUTO_RENEWAL_BEHAVIOR_DISABLE = 2

  class ChangeTypeValueValuesEnum(_messages.Enum):
    r"""Required. Type of change to make.

    Values:
      LINE_ITEM_CHANGE_TYPE_UNSPECIFIED: Sentinel value. Do not use.
      LINE_ITEM_CHANGE_TYPE_CREATE: The change is to create a new line item.
      LINE_ITEM_CHANGE_TYPE_UPDATE: The change is to update an existing line
        item.
      LINE_ITEM_CHANGE_TYPE_CANCEL: The change is to cancel an existing line
        item.
      LINE_ITEM_CHANGE_TYPE_REVERT_CANCELLATION: The change is to revert a
        cancellation.
    """
    LINE_ITEM_CHANGE_TYPE_UNSPECIFIED = 0
    LINE_ITEM_CHANGE_TYPE_CREATE = 1
    LINE_ITEM_CHANGE_TYPE_UPDATE = 2
    LINE_ITEM_CHANGE_TYPE_CANCEL = 3
    LINE_ITEM_CHANGE_TYPE_REVERT_CANCELLATION = 4

  autoRenewalBehavior = _messages.EnumField('AutoRenewalBehaviorValueValuesEnum', 1)
  changeType = _messages.EnumField('ChangeTypeValueValuesEnum', 2)
  lineItemId = _messages.StringField(3)
  newLineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 4)


class GoogleCloudCommerceConsumerProcurementV1alpha1ModifyProductsOrderRequest(_messages.Message):
  r"""Request message to update an order for non-quote products.

  Fields:
    modifications: A GoogleCloudCommerceConsumerProcurementV1alpha1ModifyProdu
      ctsOrderRequestModification attribute.
  """

  modifications = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ModifyProductsOrderRequestModification', 1, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1ModifyProductsOrderRequestModification(_messages.Message):
  r"""Modifications to make on the order.

  Enums:
    AutoRenewalBehaviorValueValuesEnum: Auto renewal behavior of the
      subscription for the update. Applied when change_type is
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE].
    ChangeTypeValueValuesEnum: Required. Type of change to make.

  Fields:
    autoRenewalBehavior: Auto renewal behavior of the subscription for the
      update. Applied when change_type is
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE].
    changeType: Required. Type of change to make.
    lineItemId: ID of the existing line item to make change to. Required when
      change type is [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE] or
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_CANCEL].
    newLineItemInfo: The line item to update to. Required when change_type is
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_CREATE] or
      [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE].
  """

  class AutoRenewalBehaviorValueValuesEnum(_messages.Enum):
    r"""Auto renewal behavior of the subscription for the update. Applied when
    change_type is [LineItemChangeType.LINE_ITEM_CHANGE_TYPE_UPDATE].

    Values:
      AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED: If unspecified, the auto renewal
        behavior will follow the default config.
      AUTO_RENEWAL_BEHAVIOR_ENABLE: Auto Renewal will be enabled on
        subscription.
      AUTO_RENEWAL_BEHAVIOR_DISABLE: Auto Renewal will be disabled on
        subscription.
    """
    AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED = 0
    AUTO_RENEWAL_BEHAVIOR_ENABLE = 1
    AUTO_RENEWAL_BEHAVIOR_DISABLE = 2

  class ChangeTypeValueValuesEnum(_messages.Enum):
    r"""Required. Type of change to make.

    Values:
      LINE_ITEM_CHANGE_TYPE_UNSPECIFIED: Sentinel value. Do not use.
      LINE_ITEM_CHANGE_TYPE_CREATE: The change is to create a new line item.
      LINE_ITEM_CHANGE_TYPE_UPDATE: The change is to update an existing line
        item.
      LINE_ITEM_CHANGE_TYPE_CANCEL: The change is to cancel an existing line
        item.
      LINE_ITEM_CHANGE_TYPE_REVERT_CANCELLATION: The change is to revert a
        cancellation.
    """
    LINE_ITEM_CHANGE_TYPE_UNSPECIFIED = 0
    LINE_ITEM_CHANGE_TYPE_CREATE = 1
    LINE_ITEM_CHANGE_TYPE_UPDATE = 2
    LINE_ITEM_CHANGE_TYPE_CANCEL = 3
    LINE_ITEM_CHANGE_TYPE_REVERT_CANCELLATION = 4

  autoRenewalBehavior = _messages.EnumField('AutoRenewalBehaviorValueValuesEnum', 1)
  changeType = _messages.EnumField('ChangeTypeValueValuesEnum', 2)
  lineItemId = _messages.StringField(3)
  newLineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 4)


class GoogleCloudCommerceConsumerProcurementV1alpha1ModifyQuoteOrderRequest(_messages.Message):
  r"""Request message for ConsumerProcurementService.ModifyOrder.

  Enums:
    AutoRenewalBehaviorValueValuesEnum: Auto renewal behavior of the
      subscription for the update. Applied when change_type is
      [QuoteChangeType.QUOTE_CHANGE_TYPE_UPDATE].
    ChangeTypeValueValuesEnum: Required. Type of change to make.

  Fields:
    autoRenewalBehavior: Auto renewal behavior of the subscription for the
      update. Applied when change_type is
      [QuoteChangeType.QUOTE_CHANGE_TYPE_UPDATE].
    changeType: Required. Type of change to make.
    newQuoteExternalName: External name of the new quote to update to.
      Required when change_type is [QuoteChangeType.QUOTE_CHANGE_TYPE_UPDATE].
  """

  class AutoRenewalBehaviorValueValuesEnum(_messages.Enum):
    r"""Auto renewal behavior of the subscription for the update. Applied when
    change_type is [QuoteChangeType.QUOTE_CHANGE_TYPE_UPDATE].

    Values:
      AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED: If unspecified, the auto renewal
        behavior will follow the default config.
      AUTO_RENEWAL_BEHAVIOR_ENABLE: Auto Renewal will be enabled on
        subscription.
      AUTO_RENEWAL_BEHAVIOR_DISABLE: Auto Renewal will be disabled on
        subscription.
    """
    AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED = 0
    AUTO_RENEWAL_BEHAVIOR_ENABLE = 1
    AUTO_RENEWAL_BEHAVIOR_DISABLE = 2

  class ChangeTypeValueValuesEnum(_messages.Enum):
    r"""Required. Type of change to make.

    Values:
      QUOTE_CHANGE_TYPE_UNSPECIFIED: Sentinel value. Do not use.
      QUOTE_CHANGE_TYPE_UPDATE: The change is to update an existing order for
        quote.
      QUOTE_CHANGE_TYPE_CANCEL: The change is to cancel an existing order for
        quote.
      QUOTE_CHANGE_TYPE_REVERT_CANCELLATION: The change is to revert a
        cancellation for an order for quote.
    """
    QUOTE_CHANGE_TYPE_UNSPECIFIED = 0
    QUOTE_CHANGE_TYPE_UPDATE = 1
    QUOTE_CHANGE_TYPE_CANCEL = 2
    QUOTE_CHANGE_TYPE_REVERT_CANCELLATION = 3

  autoRenewalBehavior = _messages.EnumField('AutoRenewalBehaviorValueValuesEnum', 1)
  changeType = _messages.EnumField('ChangeTypeValueValuesEnum', 2)
  newQuoteExternalName = _messages.StringField(3)


class GoogleCloudCommerceConsumerProcurementV1alpha1Order(_messages.Message):
  r"""Represents a purchase made by a customer on Cloud Marketplace. Creating
  an order makes sure that both the Google backend systems as well as external
  service provider's systems (if needed) allow use of purchased products and
  ensures the appropriate billing events occur. An Order can be made against
  one Product with multiple add-ons (optional) or one Quote which might
  reference multiple products. Customers typically choose a price plan for
  each Product purchased when they create an order and can change their plan
  later, if the product allows.

  Enums:
    OrderStateValueValuesEnum: Output only. The state of the order.

  Fields:
    account: The resource name of the account that this order is based on.
      Required if the creation of any products in the order requires an
      account to be present.
    cancelledLineItems: Output only. Line items that were cancelled.
    createTime: Output only. The creation timestamp.
    displayName: Required. The user-specified name of the order. Must be
      unique within a billing account.
    etag: The weak etag of the order.
    lineItems: Output only. The items being purchased.
    name: Output only. The resource name of the order. Has the form
      `billingAccounts/{billing_account}/orders/{order}`.
    orderState: Output only. The state of the order.
    provider: Provider of the products being purchased. Provider has the
      format of `providers/{provider}`.
    stateReason: Output only. An explanation for the order's state. Mainly
      used in the case of `OrderState.ORDER_STATE_CANCELLED` states to explain
      why the order is cancelled.
    updateTime: Output only. The last update timestamp.
  """

  class OrderStateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the order.

    Values:
      ORDER_STATE_UNSPECIFIED: Sentinel value. Do not use.
      ORDER_STATE_ACTIVE: The order is active.
      ORDER_STATE_CANCELLED: The order is cancelled.
      ORDER_STATE_PENDING_CANCELLATION: The order is being cancelled either by
        the user or by the system. The order stays in this state, if any
        product in the order allows use of the underlying resource until the
        end of the current billing cycle. Once the billing cycle completes,
        the resource will transition to OrderState.ORDER_STATE_CANCELLED
        state.
    """
    ORDER_STATE_UNSPECIFIED = 0
    ORDER_STATE_ACTIVE = 1
    ORDER_STATE_CANCELLED = 2
    ORDER_STATE_PENDING_CANCELLATION = 3

  account = _messages.StringField(1)
  cancelledLineItems = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItem', 2, repeated=True)
  createTime = _messages.StringField(3)
  displayName = _messages.StringField(4)
  etag = _messages.StringField(5)
  lineItems = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItem', 6, repeated=True)
  name = _messages.StringField(7)
  orderState = _messages.EnumField('OrderStateValueValuesEnum', 8)
  provider = _messages.StringField(9)
  stateReason = _messages.StringField(10)
  updateTime = _messages.StringField(11)


class GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation(_messages.Message):
  r"""OrderAllocation represents the allocation of resources within a specific
  order to a set of targets (projects). Next Id: 9

  Enums:
    StateValueValuesEnum: Output only. The state of the allocation.

  Fields:
    allocationEntry: Required. The currently effective allocation entry.
    createTime: Output only. The time when the resource was created.
    displayName: Optional. The user-specified name of the allocation. Must be
      unique within an Order if specified.
    etag: The weak etag of the order allocation, which can be optionally set.
    name: The resource name of the allocation. This field is of the form: `bil
      lingAccounts/{billing_account}/orders/{order}/orderAllocations/{order_al
      location}`.
    pendingAllocationEntry: Output only. The upcoming allocation entry.
    state: Output only. The state of the allocation.
    updateTime: Output only. The time when the resource was last updated.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the allocation.

    Values:
      STATE_UNSPECIFIED: Sentinel value. Do not use.
      STATE_PENDING_ACTIVATION: The allocation is pending activation.
      STATE_ACTIVE: The allocation is active.
      STATE_PENDING_DELETION: The allocation is pending deletion.
      STATE_CANCELLED: The allocation is cancelled. This can be caused by
        failure in creating the allocation or the associated Order being
        cancelled.
    """
    STATE_UNSPECIFIED = 0
    STATE_PENDING_ACTIVATION = 1
    STATE_ACTIVE = 2
    STATE_PENDING_DELETION = 3
    STATE_CANCELLED = 4

  allocationEntry = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocationAllocationEntry', 1)
  createTime = _messages.StringField(2)
  displayName = _messages.StringField(3)
  etag = _messages.StringField(4)
  name = _messages.StringField(5)
  pendingAllocationEntry = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocationAllocationEntry', 6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  updateTime = _messages.StringField(8)


class GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocationAllocationEntry(_messages.Message):
  r"""An allocation entry representing what resource is being allocated to
  which targets.

  Fields:
    doubleResourceValue: Represents a double value.
    int64ResourceValue: Represents an int64 value.
    stringResourceValue: Represents a string value.
    targets: Required. The targets of the allocation, only projects are
      allowed at the moment.
  """

  doubleResourceValue = _messages.FloatField(1)
  int64ResourceValue = _messages.IntegerField(2)
  stringResourceValue = _messages.StringField(3)
  targets = _messages.StringField(4, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1OrderAttribution(_messages.Message):
  r"""Determines how credits generated by this Order are assigned to specific
  targets (e.g. projects).

  Fields:
    allotments: The segments that define how total_attributable should be
      broken up. These are in priority order from highest to lowest. (The
      zero-indexed item has the highest priority.)
    intTotalAttributable: Output only. An integer amount of attributable
      resources.
    name: Output only. Resource name of the attribution configuration Format:
      billingAccounts/{billing_account}/orders/{order}/orderAttributions/{orde
      r_attribution} attribution_target references the Order parameter that
      defines the total attributable amount of this resource.
    unit: Output only. Human friendly name for what's being attributed
  """

  allotments = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAttributionAllotment', 1, repeated=True)
  intTotalAttributable = _messages.IntegerField(2)
  name = _messages.StringField(3)
  unit = _messages.StringField(4)


class GoogleCloudCommerceConsumerProcurementV1alpha1OrderAttributionAllotment(_messages.Message):
  r"""Defines a specific chunk of credits that are assigned to specific
  targets.

  Fields:
    intAllotmentAmount: An integer allotment of resources.
    targets: Targets for this allotment. Both projects and folder names are
      supported. Targets should be associated with this billing account.
      Targets not associated with this billing account are ignored. Format:
      project/{project_id} or project/{project_number} or
      folders/{folder_name}.
  """

  intAllotmentAmount = _messages.IntegerField(1)
  targets = _messages.StringField(2, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1Parameter(_messages.Message):
  r"""User-provided Parameters.

  Fields:
    name: Name of the parameter.
    value: Value of parameter.
  """

  name = _messages.StringField(1)
  value = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1ParameterValue', 2)


class GoogleCloudCommerceConsumerProcurementV1alpha1ParameterValue(_messages.Message):
  r"""A GoogleCloudCommerceConsumerProcurementV1alpha1ParameterValue object.

  Fields:
    doubleValue: Represents a double value.
    int64Value: Represents an int64 value.
    stringValue: Represents a string value.
  """

  doubleValue = _messages.FloatField(1)
  int64Value = _messages.IntegerField(2)
  stringValue = _messages.StringField(3)


class GoogleCloudCommerceConsumerProcurementV1alpha1PlaceOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.PlaceOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1PlaceOrderRequest(_messages.Message):
  r"""Request message for ConsumerProcurementService.PlaceOrder.

  Enums:
    AutoRenewalBehaviorValueValuesEnum: Optional. Auto renewal behavior of the
      subscription associated with the order.

  Fields:
    account: The resource name of the account that this order is based on. If
      this field is not specified and the creation of any products in the
      order requires an account, system will look for existing account and
      auto create one if there is no existing one.
    autoRenewalBehavior: Optional. Auto renewal behavior of the subscription
      associated with the order.
    displayName: Required. The user-specified name of the order being placed.
      Must be unique within a billing account.
    lineItemInfo: Optional. Places order for offer. Required when an offer-
      based order is being placed.
    placeProductsOrderRequest: Optional. Places order for non-quote products.
    placeQuoteOrderRequest: Optional. Places order for quote.
    provider: Required. Provider of the items being purchased. Provider has
      the format of `providers/{provider_id}`. Optional when an offer is
      specified. TODO(b/241564581) Hide provider id in the consumer API.
    requestId: Optional. A unique identifier for this request. The server will
      ignore subsequent requests that provide a duplicate request ID for at
      least 120 minutes after the first request. The request ID must be a
      valid [UUID](https://en.wikipedia.org/wiki/Universally_unique_identifier
      #Format).
    testConfig: Optional. Test configuration for the to-be-placed order.
      Placing test order is only allowed if the parent is a testing billing
      account for the service.
  """

  class AutoRenewalBehaviorValueValuesEnum(_messages.Enum):
    r"""Optional. Auto renewal behavior of the subscription associated with
    the order.

    Values:
      AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED: If unspecified, the auto renewal
        behavior will follow the default config.
      AUTO_RENEWAL_BEHAVIOR_ENABLE: Auto Renewal will be enabled on
        subscription.
      AUTO_RENEWAL_BEHAVIOR_DISABLE: Auto Renewal will be disabled on
        subscription.
    """
    AUTO_RENEWAL_BEHAVIOR_UNSPECIFIED = 0
    AUTO_RENEWAL_BEHAVIOR_ENABLE = 1
    AUTO_RENEWAL_BEHAVIOR_DISABLE = 2

  account = _messages.StringField(1)
  autoRenewalBehavior = _messages.EnumField('AutoRenewalBehaviorValueValuesEnum', 2)
  displayName = _messages.StringField(3)
  lineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 4, repeated=True)
  placeProductsOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1PlaceProductsOrderRequest', 5)
  placeQuoteOrderRequest = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1PlaceQuoteOrderRequest', 6)
  provider = _messages.StringField(7)
  requestId = _messages.StringField(8)
  testConfig = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1TestConfig', 9)


class GoogleCloudCommerceConsumerProcurementV1alpha1PlaceProductsOrderRequest(_messages.Message):
  r"""Request message to place an order for non-quote products.

  Fields:
    lineItemInfo: Required. Items to purchase within an order.
  """

  lineItemInfo = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1LineItemInfo', 1, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1PlaceQuoteOrderRequest(_messages.Message):
  r"""Request message to place an order for a quote.

  Fields:
    quoteExternalName: Required. External name of the quote to purchase.
  """

  quoteExternalName = _messages.StringField(1)


class GoogleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocationsMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.ReplaceOrderAllocations.
  """



class GoogleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocationsRequest(_messages.Message):
  r"""Request message for ConsumerProcurementService.ReplaceOrderAllocations.

  Fields:
    orderAllocations: Required. The replacement OrderAllocations. They will
      replace all existing OrderAllocations belonging to the parent. The limit
      on the number of OrderAllocations is 1000.
  """

  orderAllocations = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation', 1, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1ReplaceOrderAllocationsResponse(_messages.Message):
  r"""Message stored in the response field of the Operation returned by
  ConsumerProcurementService.ReplaceOrderAllocations.

  Fields:
    orderAllocations: The entire list of OrderAllocations of the parent Order
      after the replacement.
  """

  orderAllocations = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1alpha1OrderAllocation', 1, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1alpha1RevokeConsentRequest(_messages.Message):
  r"""Request message to revoke a consent."""


class GoogleCloudCommerceConsumerProcurementV1alpha1Subscription(_messages.Message):
  r"""Subscription information.

  Fields:
    autoRenewalEnabled: Whether auto renewal is enabled by user choice on
      current subscription. This field indicates order/subscription status
      after pending plan change is cancelled or rejected.
    endTime: The timestamp when the subscription ends, if applicable.
    startTime: The timestamp when the subscription begins, if applicable.
  """

  autoRenewalEnabled = _messages.BooleanField(1)
  endTime = _messages.StringField(2)
  startTime = _messages.StringField(3)


class GoogleCloudCommerceConsumerProcurementV1alpha1TestConfig(_messages.Message):
  r"""The test configuration for the resource.

  Fields:
    isTesting: Whether the resource is for testing or not.
  """

  isTesting = _messages.BooleanField(1)


class GoogleCloudCommerceConsumerProcurementV1alpha1UpdateOrderAllocationMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.UpdateOrderAllocation.
  """



class GoogleCloudCommerceConsumerProcurementV1mainCancelOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CancelOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1mainCreateAccountMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CreateAccount.
  """



class GoogleCloudCommerceConsumerProcurementV1mainCreateFreeTrialMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CreateFreeTrial.
  """



class GoogleCloudCommerceConsumerProcurementV1mainCreateOrderAllocationMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.CreateOrderAllocation.
  """



class GoogleCloudCommerceConsumerProcurementV1mainDeleteAccountMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.DeleteAccount.
  """



class GoogleCloudCommerceConsumerProcurementV1mainDeleteOrderAllocationMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.DeleteOrderAllocation.
  """



class GoogleCloudCommerceConsumerProcurementV1mainMigrateOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.MigrateOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1mainModifyOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.ModifyOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1mainOrderAllocation(_messages.Message):
  r"""OrderAllocation represents the allocation of resources within a specific
  order to a set of targets (projects). Next Id: 9

  Enums:
    StateValueValuesEnum: Output only. The state of the allocation.

  Fields:
    allocationEntry: Required. The currently effective allocation entry.
    createTime: Output only. The time when the resource was created.
    displayName: Optional. The user-specified name of the allocation. Must be
      unique within an Order if specified.
    etag: The weak etag of the order allocation, which can be optionally set.
    name: The resource name of the allocation. This field is of the form: `bil
      lingAccounts/{billing_account}/orders/{order}/orderAllocations/{order_al
      location}`.
    pendingAllocationEntry: Output only. The upcoming allocation entry.
    state: Output only. The state of the allocation.
    updateTime: Output only. The time when the resource was last updated.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the allocation.

    Values:
      STATE_UNSPECIFIED: Sentinel value. Do not use.
      STATE_PENDING_ACTIVATION: The allocation is pending activation.
      STATE_ACTIVE: The allocation is active.
      STATE_PENDING_DELETION: The allocation is pending deletion.
      STATE_CANCELLED: The allocation is cancelled. This can be caused by
        failure in creating the allocation or the associated Order being
        cancelled.
    """
    STATE_UNSPECIFIED = 0
    STATE_PENDING_ACTIVATION = 1
    STATE_ACTIVE = 2
    STATE_PENDING_DELETION = 3
    STATE_CANCELLED = 4

  allocationEntry = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1mainOrderAllocationAllocationEntry', 1)
  createTime = _messages.StringField(2)
  displayName = _messages.StringField(3)
  etag = _messages.StringField(4)
  name = _messages.StringField(5)
  pendingAllocationEntry = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1mainOrderAllocationAllocationEntry', 6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  updateTime = _messages.StringField(8)


class GoogleCloudCommerceConsumerProcurementV1mainOrderAllocationAllocationEntry(_messages.Message):
  r"""An allocation entry representing what resource is being allocated to
  which targets.

  Fields:
    doubleResourceValue: Represents a double value.
    int64ResourceValue: Represents an int64 value.
    stringResourceValue: Represents a string value.
    targets: Required. The targets of the allocation, only projects are
      allowed at the moment.
  """

  doubleResourceValue = _messages.FloatField(1)
  int64ResourceValue = _messages.IntegerField(2)
  stringResourceValue = _messages.StringField(3)
  targets = _messages.StringField(4, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1mainPlaceOrderMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.PlaceOrder.
  """



class GoogleCloudCommerceConsumerProcurementV1mainReplaceOrderAllocationsMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.ReplaceOrderAllocations.
  """



class GoogleCloudCommerceConsumerProcurementV1mainReplaceOrderAllocationsResponse(_messages.Message):
  r"""Message stored in the response field of the Operation returned by
  ConsumerProcurementService.ReplaceOrderAllocations.

  Fields:
    orderAllocations: The entire list of OrderAllocations of the parent Order
      after the replacement.
  """

  orderAllocations = _messages.MessageField('GoogleCloudCommerceConsumerProcurementV1mainOrderAllocation', 1, repeated=True)


class GoogleCloudCommerceConsumerProcurementV1mainUpdateOrderAllocationMetadata(_messages.Message):
  r"""Message stored in the metadata field of the Operation returned by
  ConsumerProcurementService.UpdateOrderAllocation.
  """



class GoogleLongrunningOperation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('GoogleRpcStatus', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class GoogleRpcStatus(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class GoogleTypeMoney(_messages.Message):
  r"""Represents an amount of money with its currency type.

  Fields:
    currencyCode: The three-letter currency code defined in ISO 4217.
    nanos: Number of nano (10^-9) units of the amount. The value must be
      between -999,999,999 and +999,999,999 inclusive. If `units` is positive,
      `nanos` must be positive or zero. If `units` is zero, `nanos` can be
      positive, zero, or negative. If `units` is negative, `nanos` must be
      negative or zero. For example $-1.75 is represented as `units`=-1 and
      `nanos`=-750,000,000.
    units: The whole units of the amount. For example if `currencyCode` is
      `"USD"`, then 1 unit is one US dollar.
  """

  currencyCode = _messages.StringField(1)
  nanos = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  units = _messages.IntegerField(3)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Check autopilot compatibility command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


@base.Hidden
class CheckAutopliotCompatibility(base.ListCommand):
  """Check autopilot compatibility of a running cluster."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To check autopilot compatibility of an existing cluster, run:

            $ {command} sample-cluster
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('name', help='The name of this cluster.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    adapter = self.context['api_adapter']
    location_get = self.context['location_get']
    location = location_get(args)

    def sort_key(issue):
      return (issue.type, issue.issueId)

    issues = adapter.CheckAutopilotCompatibility(
        adapter.ParseCluster(args.name, location))
    issues.issues = sorted(issues.issues, key=sort_key)

    return issues.issues




# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Class for representing various changes to a Configuration."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import abc
import collections
import copy
import json

from googlecloudsdk.api_lib.run import container_resource
from googlecloudsdk.api_lib.run import job
from googlecloudsdk.api_lib.run import k8s_object
from googlecloudsdk.api_lib.run import revision
from googlecloudsdk.api_lib.run import service
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import name_generator
from googlecloudsdk.command_lib.run import platforms
from googlecloudsdk.command_lib.run import secrets_mapping
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.args import repeated
import six


class ConfigChanger(six.with_metaclass(abc.ABCMeta, object)):
  """An abstract class representing configuration changes."""

  def __init__(self, adjusts_template):
    self._adjusts_template = adjusts_template

  @property
  def adjusts_template(self):
    return self._adjusts_template

  @abc.abstractmethod
  def Adjust(self, resource):
    """Adjust the given Service configuration.

    Args:
      resource: the k8s_object to adjust.

    Returns:
      A k8s_object that reflects applying the requested update.
      May be resource after a mutation or a different object.
    """
    return resource


def WithChanges(resource, changes):
  """Apply ConfigChangers to resource.

  It's undefined whether the input resource is modified.

  Args:
    resource: KubernetesObject, probably a Service.
    changes: List of ConfigChangers.

  Returns:
    Changed resource.
  """
  for config_change in changes:
    resource = config_change.Adjust(resource)
  return resource


def AdjustsTemplate(changes):
  """Returns True if there's any template-level changes."""
  return any([c.adjusts_template for c in changes])


class LabelChanges(ConfigChanger):
  """Represents the user intent to modify metadata labels."""

  LABELS_NOT_ALLOWED_IN_REVISION = ([service.ENDPOINT_VISIBILITY])

  def __init__(self, diff, copy_to_revision=True):
    super(LabelChanges, self).__init__(adjusts_template=copy_to_revision)
    self._diff = diff
    self._copy_to_revision = copy_to_revision

  def Adjust(self, resource):
    # Currently assumes all "system"-owned labels are applied by the control
    # plane and it's ok for us to clear them on the client.
    update_result = self._diff.Apply(
        k8s_object.Meta(resource.MessagesModule()).LabelsValue,
        resource.metadata.labels)
    maybe_new_labels = update_result.GetOrNone()
    if maybe_new_labels:
      resource.metadata.labels = maybe_new_labels
      # For job, resource.template points to task template which has no
      # metadata. Use job specific execution_template instead.
      template = resource.execution_template if hasattr(
          resource, 'execution_template') else resource.template
      if self._copy_to_revision and hasattr(template, 'labels'):
        # Service labels are the source of truth and *overwrite* revision labels
        # See go/run-labels-prd for deets.
        # However, we need to preserve the nonce if there is one.
        nonce = template.labels.get(revision.NONCE_LABEL)
        template.metadata.labels = copy.deepcopy(maybe_new_labels)
        for label_to_remove in self.LABELS_NOT_ALLOWED_IN_REVISION:
          if label_to_remove in template.labels:
            del template.labels[label_to_remove]
        if nonce:
          template.labels[revision.NONCE_LABEL] = nonce
    return resource


class JobNonceChange(ConfigChanger):
  """Adds a new nonce to the job template, for forcing an image pull."""

  def __init__(self):
    super(JobNonceChange, self).__init__(adjusts_template=True)

  def Adjust(self, resource):
    resource.execution_template.labels[
        job.NONCE_LABEL] = name_generator.GenerateName(
            3, separator='_')

    return resource


class ReplaceJobChange(ConfigChanger):
  """Represents the user intent to replace the job."""

  def __init__(self, new_job):
    super(ReplaceJobChange, self).__init__(adjusts_template=False)
    self._job = new_job

  def Adjust(self, resource):
    """Returns a replacement for resource.

    The returned job is the job provided to the constructor. If
    resource.metadata.resourceVersion is not empty, has metadata.resourceVersion
    of returned job set to this value.

    Args:
      resource: job.Job, The job to adjust.
    """
    if resource.metadata.resourceVersion:
      self._job.metadata.resourceVersion = resource.metadata.resourceVersion
    return self._job


class ReplaceServiceChange(ConfigChanger):
  """Represents the user intent to replace the service."""

  def __init__(self, new_service):
    super(ReplaceServiceChange, self).__init__(adjusts_template=False)
    self._service = new_service

  def Adjust(self, resource):
    """Returns a replacement for resource.

    The returned service is the service provided to the constructor. If
    resource.metadata.resourceVersion is not empty, has metadata.resourceVersion
    of returned service set to this value.

    Args:
      resource: service.Service, The service to adjust.
    """
    if resource.metadata.resourceVersion:
      self._service.metadata.resourceVersion = resource.metadata.resourceVersion
      # Knative will complain if you try to edit (incl remove) serving annots.
      # So replicate them here.
      for k, v in resource.annotations.items():
        if k.startswith(k8s_object.SERVING_GROUP):
          self._service.annotations[k] = v
    return self._service


class EndpointVisibilityChange(LabelChanges):
  """Represents the user intent to modify the endpoint visibility.

  Only applies to Cloud Run for Anthos.
  """

  def __init__(self, endpoint_visibility):
    """Determine label changes for modifying endpoint visibility.

    Args:
      endpoint_visibility: bool, True if Cloud Run on GKE service should only be
        addressable from within the cluster. False if it should be publicly
        addressable.
    """
    if endpoint_visibility:
      diff = labels_util.Diff(
          additions={service.ENDPOINT_VISIBILITY: service.CLUSTER_LOCAL})
    else:
      diff = labels_util.Diff(subtractions=[service.ENDPOINT_VISIBILITY])
    # Don't copy this label to the revision because it's not supported there.
    # See b/154664962.
    super(EndpointVisibilityChange, self).__init__(diff, False)


class SetAnnotationChange(ConfigChanger):
  """Represents the user intent to set an annotation."""

  def __init__(self, key, value):
    super(SetAnnotationChange, self).__init__(adjusts_template=False)
    self._key = key
    self._value = value

  def Adjust(self, resource):
    resource.annotations[self._key] = self._value
    return resource


class DeleteAnnotationChange(ConfigChanger):
  """Represents the user intent to delete an annotation."""

  def __init__(self, key):
    super(DeleteAnnotationChange, self).__init__(adjusts_template=False)
    self._key = key

  def Adjust(self, resource):
    annotations = resource.annotations
    if self._key in annotations:
      del annotations[self._key]
    return resource


class SetTemplateAnnotationChange(ConfigChanger):
  """Represents the user intent to set a template annotation."""

  def __init__(self, key, value):
    super(SetTemplateAnnotationChange, self).__init__(adjusts_template=True)
    self._key = key
    self._value = value

  def Adjust(self, resource):
    resource.template.annotations[self._key] = self._value
    return resource


class DeleteTemplateAnnotationChange(ConfigChanger):
  """Represents the user intent to delete a template annotation."""

  def __init__(self, key):
    super(DeleteTemplateAnnotationChange, self).__init__(adjusts_template=True)
    self._key = key

  def Adjust(self, resource):
    annotations = resource.template.annotations
    if self._key in annotations:
      del annotations[self._key]
    return resource


class SetLaunchStageAnnotationChange(ConfigChanger):
  """Sets a VPC connector annotation on the service."""

  def __init__(self, launch_stage):
    super(SetLaunchStageAnnotationChange, self).__init__(adjusts_template=False)
    self._launch_stage = launch_stage

  def Adjust(self, resource):
    if self._launch_stage == base.ReleaseTrack.GA:
      return resource
    else:
      resource.annotations[
          k8s_object.LAUNCH_STAGE_ANNOTATION] = self._launch_stage.id
      return resource


class SetClientNameAndVersionAnnotationChange(ConfigChanger):
  """Sets the client name and version annotations."""

  def __init__(self, client_name, client_version, set_on_template=True):
    super(SetClientNameAndVersionAnnotationChange,
          self).__init__(adjusts_template=set_on_template)
    self._client_name = client_name
    self._client_version = client_version
    self._set_on_template = set_on_template

  def Adjust(self, resource):
    if self._client_name is not None:
      resource.annotations[
          k8s_object.CLIENT_NAME_ANNOTATION] = self._client_name
      if self._set_on_template and hasattr(resource.template, 'annotations'):
        resource.template.annotations[
            k8s_object.CLIENT_NAME_ANNOTATION] = self._client_name
    if self._client_version is not None:
      resource.annotations[
          k8s_object.CLIENT_VERSION_ANNOTATION] = self._client_version
      if self._set_on_template and hasattr(resource.template, 'annotations'):
        resource.template.annotations[
            k8s_object.CLIENT_VERSION_ANNOTATION] = self._client_version
    return resource


class SandboxChange(ConfigChanger):
  """Sets a sandbox annotation on the service."""

  def __init__(self, sandbox):
    super(SandboxChange, self).__init__(adjusts_template=True)
    self._sandbox = sandbox

  def Adjust(self, resource):
    resource.template.annotations[
        container_resource.SANDBOX_ANNOTATION] = self._sandbox
    return resource


class VpcConnectorChange(ConfigChanger):
  """Sets a VPC connector annotation on the service."""

  def __init__(self, connector_name):
    super(VpcConnectorChange, self).__init__(adjusts_template=True)
    self._connector_name = connector_name

  def Adjust(self, resource):
    resource.template.annotations[
        container_resource.VPC_ACCESS_ANNOTATION] = self._connector_name
    return resource


class ClearVpcConnectorChange(ConfigChanger):
  """Clears a VPC connector annotation on the service."""

  def __init__(self):
    super(ClearVpcConnectorChange, self).__init__(adjusts_template=True)

  def Adjust(self, resource):
    annotations = resource.template.annotations
    if container_resource.VPC_ACCESS_ANNOTATION in annotations:
      del annotations[container_resource.VPC_ACCESS_ANNOTATION]
    if container_resource.EGRESS_SETTINGS_ANNOTATION in annotations:
      del annotations[container_resource.EGRESS_SETTINGS_ANNOTATION]
    return resource


class ImageChange(ConfigChanger):
  """A Cloud Run container deployment."""

  deployment_type = 'container'

  def __init__(self, image):
    super(ImageChange, self).__init__(adjusts_template=True)
    self.image = image

  def Adjust(self, resource):
    resource.annotations[container_resource.USER_IMAGE_ANNOTATION] = (
        self.image)
    if hasattr(resource.template, 'annotations'):
      resource.template.annotations[
          container_resource.USER_IMAGE_ANNOTATION] = (
              self.image)
    resource.image = self.image
    return resource


def _PruneMapping(mapping, keys_to_remove, clear_others):
  if clear_others:
    mapping.clear()
  else:
    for var_or_path in keys_to_remove:
      if var_or_path in mapping:
        del mapping[var_or_path]


def _PruneManagedVolumeMapping(volumes, volume_mounts, removes, clear_others):
  """Remove the specified volume mappings from the config."""
  if clear_others:
    volume_mounts.clear()
  else:
    for remove in removes:
      mount, path = remove.rsplit('/', 1)
      if mount in volume_mounts:
        new_paths = []
        for key_to_path in volumes[volume_mounts[mount]].items:
          if path != key_to_path.path:
            new_paths.append(key_to_path)
        if not new_paths:
          # there are no more versions in the volume
          del volume_mounts[mount]
        else:
          volumes[volume_mounts[mount]].items = new_paths


def _CopyToNewVolume(resource, volume_name, mount_point, volume_source, volumes,
                     volume_mounts):
  """Copies existing volume to volume with a new name."""
  new_volume_name = _UniqueVolumeName(volume_source.secretName,
                                      resource.template.volumes)
  try:
    volume_mounts[mount_point] = new_volume_name
  except KeyError:
    raise exceptions.ConfigurationError(
        'Cannot update mount [{}] because its mounted volume '
        'is of a different source type.'.format(mount_point))
    # the volume does not exist so we need a new one
  new_paths = {item.path for item in volume_source.items}
  old_volume = volumes[volume_name]
  for item in old_volume.items:
    if item.path not in new_paths:
      volume_source.items.append(item)
  volumes[new_volume_name] = volume_source
  return new_volume_name


class EnvVarLiteralChanges(ConfigChanger):
  """Represents the user intent to modify environment variables string literals.
  """

  def __init__(self, updates, removes, clear_others):
    """Initialize a new EnvVarLiteralChanges object.

    Args:
      updates: {str, str}, Update env var names and values.
      removes: [str], List of env vars to remove.
      clear_others: bool, If true, clear all non-updated env vars.
    """
    super(EnvVarLiteralChanges, self).__init__(adjusts_template=True)
    self._updates = updates
    self._removes = removes
    self._clear_others = clear_others

  def Adjust(self, resource):
    """Mutates the given config's env vars to match the desired changes.

    Args:
      resource: k8s_object to adjust

    Returns:
      The adjusted resource

    Raises:
      ConfigurationError if there's an attempt to replace the source of an
        existing environment variable whose source is of a different type
        (e.g. env var's secret source can't be replaced with a config map
        source).
    """
    _PruneMapping(resource.template.env_vars.literals, self._removes,
                  self._clear_others)

    try:
      resource.template.env_vars.literals.update(self._updates)
    except KeyError as e:
      raise exceptions.ConfigurationError(
          'Cannot update environment variable [{}] to string literal '
          'because it has already been set with a different type.'.format(
              e.args[0]))
    return resource


class SecretEnvVarChanges(ConfigChanger):
  """Represents the user intent to modify environment variable secrets."""

  def __init__(self, updates, removes, clear_others):
    """Initialize a new SecretEnvVarChanges object.

    Args:
      updates: {str: ReachableSecret}, Update env var names and values.
      removes: [str], List of env vars to remove.
      clear_others: bool, If true, clear all non-updated env vars.

    Raises:
      ConfigurationError if a key hasn't been provided for a source.
    """
    super(SecretEnvVarChanges, self).__init__(adjusts_template=True)
    self._updates = updates
    self._removes = removes
    self._clear_others = clear_others

  def Adjust(self, resource):
    """Mutates the given config's env vars to match the desired changes.

    Args:
      resource: k8s_object to adjust

    Returns:
      The adjusted resource

    Raises:
      ConfigurationError if there's an attempt to replace the source of an
        existing environment variable whose source is of a different type
        (e.g. env var's secret source can't be replaced with a config map
        source).
    """
    env_vars = resource.template.env_vars.secrets
    _PruneMapping(env_vars, self._removes, self._clear_others)

    for name, reachable_secret in self._updates.items():
      try:
        env_vars[name] = reachable_secret.AsEnvVarSource(resource)
      except KeyError:
        raise exceptions.ConfigurationError(
            'Cannot update environment variable [{}] to the given type '
            'because it has already been set with a different type.'.format(
                name))
    secrets_mapping.PruneAnnotation(resource)
    return resource


class ConfigMapEnvVarChanges(ConfigChanger):
  """Represents the user intent to modify environment variable config maps."""

  def __init__(self, updates, removes, clear_others):
    """Initialize a new ConfigMapEnvVarChanges object.

    Args:
      updates: {str, str}, Update env var names and values.
      removes: [str], List of env vars to remove.
      clear_others: bool, If true, clear all non-updated env vars.

    Raises:
      ConfigurationError if a key hasn't been provided for a source.
    """
    super(ConfigMapEnvVarChanges, self).__init__(adjusts_template=True)
    self._updates = {}
    for name, v in updates.items():
      # Split the given values into 2 parts:
      #    [env var source name, source data item key]
      value = v.split(':', 1)
      if len(value) < 2:
        value.append(self._OmittedSecretKeyDefault(name))
      self._updates[name] = value
    self._removes = removes
    self._clear_others = clear_others

  def _OmittedSecretKeyDefault(self, name):
    if platforms.IsManaged():
      return 'latest'
    raise exceptions.ConfigurationError(
        'Missing required item key for environment variable [{}].'.format(name))

  def Adjust(self, resource):
    """Mutates the given config's env vars to match the desired changes.

    Args:
      resource: k8s_object to adjust

    Returns:
      The adjusted resource

    Raises:
      ConfigurationError if there's an attempt to replace the source of an
        existing environment variable whose source is of a different type
        (e.g. env var's secret source can't be replaced with a config map
        source).
    """
    env_vars = resource.template.env_vars.config_maps
    _PruneMapping(env_vars, self._removes, self._clear_others)

    for name, (source_name, source_key) in self._updates.items():
      try:
        env_vars[name] = self._MakeEnvVarSource(resource.MessagesModule(),
                                                source_name, source_key)
      except KeyError:
        raise exceptions.ConfigurationError(
            'Cannot update environment variable [{}] to the given type '
            'because it has already been set with a different type.'.format(
                name))
    return resource

  def _MakeEnvVarSource(self, messages, name, key):
    return messages.EnvVarSource(
        configMapKeyRef=messages.ConfigMapKeySelector(name=name, key=key))


class ResourceChanges(ConfigChanger):
  """Represents the user intent to update resource limits."""

  def __init__(self, memory=None, cpu=None):
    super(ResourceChanges, self).__init__(adjusts_template=True)
    self._memory = memory
    self._cpu = cpu

  def Adjust(self, resource):
    """Mutates the given config's resource limits to match what's desired."""
    if self._memory is not None:
      resource.template.resource_limits['memory'] = self._memory
    if self._cpu is not None:
      resource.template.resource_limits['cpu'] = self._cpu
    return resource


class CloudSQLChanges(ConfigChanger):
  """Represents the intent to update the Cloug SQL instances."""

  def __init__(self, project, region, args):
    """Initializes the intent to update the Cloud SQL instances.

    Args:
      project: Project to use as the default project for Cloud SQL instances.
      region: Region to use as the default region for Cloud SQL instances
      args: Args to the command.
    """
    super(CloudSQLChanges, self).__init__(adjusts_template=True)
    self._project = project
    self._region = region
    self._args = args

  # Here we are a proxy through to the actual args to set some extra augmented
  # information on each one, so each cloudsql instance gets the region and
  # project.
  @property
  def add_cloudsql_instances(self):
    return self._AugmentArgs('add_cloudsql_instances')

  @property
  def remove_cloudsql_instances(self):
    return self._AugmentArgs('remove_cloudsql_instances')

  @property
  def set_cloudsql_instances(self):
    return self._AugmentArgs('set_cloudsql_instances')

  @property
  def clear_cloudsql_instances(self):
    return getattr(self._args, 'clear_cloudsql_instances', None)

  def _AugmentArgs(self, arg_name):
    val = getattr(self._args, arg_name, None)
    if val is None:
      return None
    return [self._Augment(i) for i in val]

  def Adjust(self, resource):

    def GetCurrentInstances():
      annotation_val = resource.template.annotations.get(
          container_resource.CLOUDSQL_ANNOTATION)
      if annotation_val:
        return annotation_val.split(',')
      return []

    instances = repeated.ParsePrimitiveArgs(self, 'cloudsql-instances',
                                            GetCurrentInstances)
    if instances is not None:
      resource.template.annotations[
          container_resource.CLOUDSQL_ANNOTATION] = ','.join(instances)
    return resource

  def _Augment(self, instance_str):
    instance = instance_str.split(':')
    if len(instance) == 3:
      ret = tuple(instance)
    elif len(instance) == 1:
      if not self._project:
        raise exceptions.CloudSQLError(
            'To specify a Cloud SQL instance by plain name, you must specify a '
            'project.')
      if not self._region:
        raise exceptions.CloudSQLError(
            'To specify a Cloud SQL instance by plain name, you must be '
            'deploying to a managed Cloud Run region.')
      ret = self._project, self._region, instance[0]
    else:
      raise exceptions.CloudSQLError(
          'Malformed CloudSQL instance string: {}'.format(instance_str))
    return ':'.join(ret)


class ConcurrencyChanges(ConfigChanger):
  """Represents the user intent to update concurrency preference."""

  def __init__(self, concurrency):
    super(ConcurrencyChanges, self).__init__(adjusts_template=True)
    self._concurrency = None if concurrency == 'default' else int(concurrency)

  def Adjust(self, resource):
    """Mutates the given config's resource limits to match what's desired."""
    resource.template.concurrency = self._concurrency
    return resource


class TimeoutChanges(ConfigChanger):
  """Represents the user intent to update request duration."""

  def __init__(self, timeout):
    super(TimeoutChanges, self).__init__(adjusts_template=True)
    self._timeout = timeout

  def Adjust(self, resource):
    """Mutates the given config's timeout to match what's desired."""
    resource.template.timeout = self._timeout
    return resource


class ServiceAccountChanges(ConfigChanger):
  """Represents the user intent to change service account for the revision."""

  def __init__(self, service_account):
    super(ServiceAccountChanges, self).__init__(adjusts_template=True)
    self._service_account = service_account

  def Adjust(self, resource):
    """Mutates the given config's service account to match what's desired."""
    resource.template.service_account = self._service_account
    return resource


_MAX_RESOURCE_NAME_LENGTH = 63


class RevisionNameChanges(ConfigChanger):
  """Represents the user intent to change revision name."""

  def __init__(self, revision_suffix):
    super(RevisionNameChanges, self).__init__(adjusts_template=True)
    self._revision_suffix = revision_suffix

  def Adjust(self, resource):
    """Mutates the given config's revision name to match what's desired."""
    max_prefix_length = (
        _MAX_RESOURCE_NAME_LENGTH - len(self._revision_suffix) - 1)
    resource.template.name = '{}-{}'.format(resource.name[:max_prefix_length],
                                            self._revision_suffix)
    return resource


def _GenerateVolumeName(prefix):
  """Randomly generated name with the given prefix."""
  return name_generator.GenerateName(sections=3, separator='-', prefix=prefix)


def _UniqueVolumeName(source_name, existing_volumes):
  """Generate unique volume name.

  The names that connect volumes and mounts must be unique even if their
  source volume names match.

  Args:
    source_name: (Potentially clashing) name.
    existing_volumes: Names in use.

  Returns:
    Unique name.
  """
  volume_name = None
  while volume_name is None or volume_name in existing_volumes:
    volume_name = _GenerateVolumeName(source_name)
  return volume_name


def _PruneVolumes(volume_mounts, volumes):
  """Delete all volumes no longer being mounted.

  Args:
    volume_mounts: resource.template.volume_mounts
    volumes: resource.template.volumes
  """
  for volume in list(volumes):
    if volume not in volume_mounts.values():
      del volumes[volume]


class SecretVolumeChanges(ConfigChanger):
  """Represents the user intent to change volumes with secret source types."""

  def __init__(self, updates, removes, clear_others):
    """Initialize a new SecretVolumeChanges object.

    Args:
      updates: {str: ReachableSecret}, Update mount path and volume fields.
      removes: [str], List of mount paths to remove.
      clear_others: bool, If true, clear all non-updated volumes and mounts of
        the given [volume_type].
    """
    super(SecretVolumeChanges, self).__init__(adjusts_template=True)
    self._updates = updates
    self._removes = removes
    self._clear_others = clear_others

  def _UpdateManagedVolumes(self, resource, volume_mounts, volumes):
    """Update volumes for Cloud Run. Ensure only one secret per directory."""
    new_volumes = {}
    volumes_to_mounts = collections.defaultdict(list)
    for path, vol in volume_mounts.items():
      volumes_to_mounts[vol].append(path)

    for file_path, reachable_secret in self._updates.items():
      mount_point = file_path.rsplit('/', 1)[0]
      if mount_point in new_volumes:
        if new_volumes[mount_point].secretName != reachable_secret.secret_name:
          # we don't support subpaths in managed so if there's a second
          # secret in the same directory, error.
          raise exceptions.ConfigurationError(
              'Cannot update secret at [{}] because a different secret is '
              'already mounted in the same directory.'.format(file_path))
        reachable_secret.AppendToSecretVolumeSource(resource,
                                                    new_volumes[mount_point])
      else:
        new_volumes[mount_point] = reachable_secret.AsSecretVolumeSource(
            resource)

    for mount_point, volume_source in new_volumes.items():
      if mount_point in volume_mounts:
        volume_name = volume_mounts[mount_point]
        if len(volumes_to_mounts[volume_name]) > 1:
          # the volume is used by more than one path, let's separate it into a
          # separate volume
          volumes_to_mounts[volume_name].remove(mount_point)
          new_name = _CopyToNewVolume(resource, volume_name, mount_point,
                                      volume_source, volumes, volume_mounts)
          volumes_to_mounts[new_name].append(mount_point)
          continue
        else:
          volume = volumes[volume_name]
          if volume.secretName != volume_source.secretName:
            # only allow replacing the secret if all versions are replaced
            existing_paths = {item.path for item in volume.items}
            new_paths = {item.path for item in volume_source.items}
            if not existing_paths.issubset(new_paths):
              raise exceptions.ConfigurationError(
                  'Multiple secrets are specified for directory [{}]. Cloud Run '
                  'only supports one secret per directory'.format(mount_point))
          else:
            # we need to merge the two
            new_paths = {item.path for item in volume_source.items}
            for item in volume.items:
              # copy over existing paths that are not overridden
              if item.path not in new_paths:
                volume_source.items.append(item)
      else:
        volume_name = _UniqueVolumeName(volume_source.secretName,
                                        resource.template.volumes)
        try:
          volume_mounts[mount_point] = volume_name
        except KeyError:
          raise exceptions.ConfigurationError(
              'Cannot update mount [{}] because its mounted volume '
              'is of a different source type.'.format(mount_point))
          # the volume does not exist so we need a new one
      volumes[volume_name] = volume_source

  def Adjust(self, resource):
    """Mutates the given config's volumes to match the desired changes.

    Args:
      resource: k8s_object to adjust

    Returns:
      The adjusted resource

    Raises:
      ConfigurationError if there's an attempt to replace the volume a mount
        points to whose existing volume has a source of a different type than
        the new volume (e.g. mount that points to a volume with a secret source
        can't be replaced with a volume that has a config map source).
    """
    volume_mounts = resource.template.volume_mounts.secrets
    volumes = resource.template.volumes.secrets

    if platforms.IsManaged():
      _PruneManagedVolumeMapping(volumes, volume_mounts, self._removes,
                                 self._clear_others)
    else:
      removes = self._removes
      _PruneMapping(volume_mounts, removes, self._clear_others)
    if platforms.IsManaged():
      self._UpdateManagedVolumes(resource, volume_mounts, volumes)
    else:
      for file_path, reachable_secret in self._updates.items():
        volume_name = _UniqueVolumeName(reachable_secret.secret_name,
                                        resource.template.volumes)

        # volume_mounts is a special mapping that filters for the current kind
        # of mount and KeyErrors on existing keys with other types.
        try:
          mount_point = file_path
          volume_mounts[mount_point] = volume_name
        except KeyError:
          raise exceptions.ConfigurationError(
              'Cannot update mount [{}] because its mounted volume '
              'is of a different source type.'.format(file_path))
        volumes[volume_name] = reachable_secret.AsSecretVolumeSource(resource)

    _PruneVolumes(volume_mounts, volumes)
    secrets_mapping.PruneAnnotation(resource)
    return resource


class ConfigMapVolumeChanges(ConfigChanger):
  """Represents the user intent to change volumes with config map source types.
  """

  def __init__(self, updates, removes, clear_others):
    """Initialize a new ConfigMapVolumeChanges object.

    Args:
      updates: {str, [str, str]}, Update mount path and volume fields.
      removes: [str], List of mount paths to remove.
      clear_others: bool, If true, clear all non-updated volumes and mounts of
        the given [volume_type].
    """
    super(ConfigMapVolumeChanges, self).__init__(adjusts_template=True)
    self._updates = {}
    for k, v in updates.items():
      # Split the given values into 2 parts:
      #    [volume source name, data item key]
      update_value = v.split(':', 1)
      # Pad with None if no data item key specified
      if len(update_value) < 2:
        update_value.append(None)
      self._updates[k] = update_value
    self._removes = removes
    self._clear_others = clear_others

  def Adjust(self, resource):
    """Mutates the given config's volumes to match the desired changes.

    Args:
      resource: k8s_object to adjust

    Returns:
      The adjusted resource

    Raises:
      ConfigurationError if there's an attempt to replace the volume a mount
        points to whose existing volume has a source of a different type than
        the new volume (e.g. mount that points to a volume with a secret source
        can't be replaced with a volume that has a config map source).
    """
    volume_mounts = resource.template.volume_mounts.config_maps
    volumes = resource.template.volumes.config_maps

    _PruneMapping(volume_mounts, self._removes, self._clear_others)

    for path, (source_name, source_key) in self._updates.items():
      volume_name = _UniqueVolumeName(source_name, resource.template.volumes)

      # volume_mounts is a special mapping that filters for the current kind
      # of mount and KeyErrors on existing keys with other types.
      try:
        volume_mounts[path] = volume_name
      except KeyError:
        raise exceptions.ConfigurationError(
            'Cannot update mount [{}] because its mounted volume '
            'is of a different source type.'.format(path))
      volumes[volume_name] = self._MakeVolumeSource(resource.MessagesModule(),
                                                    source_name, source_key)

    _PruneVolumes(volume_mounts, volumes)

    return resource

  def _MakeVolumeSource(self, messages, name, key=None):
    source = messages.ConfigMapVolumeSource(name=name)
    if key is not None:
      source.items.append(messages.KeyToPath(key=key, path=key))
    return source


class NoTrafficChange(ConfigChanger):
  """Represents the user intent to block traffic for a new revision."""

  def __init__(self):
    super(NoTrafficChange, self).__init__(adjusts_template=False)

  def Adjust(self, resource):
    """Removes LATEST from the services traffic assignments."""
    if resource.configuration:
      raise exceptions.UnsupportedOperationError(
          'This service is using an old version of Cloud Run for Anthos '
          'that does not support traffic features. Please upgrade to 0.8 '
          'or later.')

    if not resource.generation:
      raise exceptions.ConfigurationError(
          '--no-traffic not supported when creating a new service.')

    resource.spec_traffic.ZeroLatestTraffic(
        resource.status.latestReadyRevisionName)
    return resource


class TrafficChanges(ConfigChanger):
  """Represents the user intent to change a service's traffic assignments."""

  def __init__(self,
               new_percentages,
               by_tag=False,
               tags_to_update=None,
               tags_to_remove=None,
               clear_other_tags=False):
    super(TrafficChanges, self).__init__(adjusts_template=False)
    self._new_percentages = new_percentages
    self._by_tag = by_tag
    self._tags_to_update = tags_to_update or {}
    self._tags_to_remove = tags_to_remove or []
    self._clear_other_tags = clear_other_tags

  def Adjust(self, resource):
    """Mutates the given service's traffic assignments."""
    if self._tags_to_update or self._tags_to_remove or self._clear_other_tags:
      resource.spec_traffic.UpdateTags(self._tags_to_update,
                                       self._tags_to_remove,
                                       self._clear_other_tags)
    if self._new_percentages:
      if self._by_tag:
        tag_to_key = resource.spec_traffic.TagToKey()
        percentages = {}
        for tag in self._new_percentages:
          try:
            percentages[tag_to_key[tag]] = self._new_percentages[tag]
          except KeyError:
            raise exceptions.ConfigurationError(
                'There is no revision tagged with [{}] in the traffic allocation for [{}].'
                .format(tag, resource.name))
      else:
        percentages = self._new_percentages
      resource.spec_traffic.UpdateTraffic(percentages)
    return resource


class TagOnDeployChange(ConfigChanger):
  """The intent to provide a tag for the revision you're currently deploying."""

  def __init__(self, tag):
    super(TagOnDeployChange, self).__init__(adjusts_template=False)
    self._tag = tag

  def Adjust(self, resource):
    """Gives the revision that's being created the given tag."""
    tags_to_update = {self._tag: resource.template.name}
    resource.spec_traffic.UpdateTags(tags_to_update, [], False)
    return resource


class ContainerCommandChange(ConfigChanger):
  """Represents the user intent to change the 'command' for the container."""

  def __init__(self, command):
    super(ContainerCommandChange, self).__init__(adjusts_template=True)
    self._commands = command

  def Adjust(self, resource):
    resource.template.container.command = self._commands
    return resource


class ContainerArgsChange(ConfigChanger):
  """Represents the user intent to change the 'args' for the container."""

  def __init__(self, args):
    super(ContainerArgsChange, self).__init__(adjusts_template=True)
    self._args = args

  def Adjust(self, resource):
    resource.template.container.args = self._args
    return resource


_HTTP2_NAME = 'h2c'
_DEFAULT_PORT = 8080


class ContainerPortChange(ConfigChanger):
  """Represents the user intent to change the port name and/or number."""

  def __init__(self, port=None, use_http2=None):
    """Initialize a ContainerPortChange.

    Args:
      port: str, the port number to set the port to, "default" to unset the
        containerPort field, or None to not modify the port number.
      use_http2: bool, True to set the port name for http/2, False to unset it,
        or None to not modify the port name.
    """
    super(ContainerPortChange, self).__init__(adjusts_template=True)
    self._port = port
    self._http2 = use_http2

  def Adjust(self, resource):
    """Modify an existing ContainerPort or create a new one."""
    port_msg = (
        resource.template.container.ports[0]
        if resource.template.container.ports else
        resource.MessagesModule().ContainerPort())
    # Set port to given value or clear field
    if self._port == 'default':
      port_msg.reset('containerPort')
    elif self._port is not None:
      port_msg.containerPort = int(self._port)
    # Set name for http/2 or clear field
    if self._http2:
      port_msg.name = _HTTP2_NAME
    elif self._http2 is not None:
      port_msg.reset('name')
    # A port number must be specified
    if port_msg.name and not port_msg.containerPort:
      port_msg.containerPort = _DEFAULT_PORT

    # Use the ContainerPort iff it's not empty
    if port_msg.containerPort:
      resource.template.container.ports = [port_msg]
    else:
      resource.template.container.reset('ports')
    return resource


class SpecChange(ConfigChanger):
  """Represents the user intent to update field in the resource's spec."""

  def __init__(self, field, value):
    super(SpecChange, self).__init__(adjusts_template=True)
    self._field = field
    self._value = value

  def Adjust(self, resource):
    setattr(resource.spec, self._field, self._value)
    return resource


class ExecutionTemplateSpecChange(ConfigChanger):
  """Represents the intent to update field in an execution template's spec."""

  def __init__(self, field, value):
    super(ExecutionTemplateSpecChange, self).__init__(adjusts_template=True)
    self._field = field
    self._value = value

  def Adjust(self, resource):
    setattr(resource.execution_template.spec, self._field, self._value)
    return resource


class JobMaxRetriesChange(ConfigChanger):
  """Represents the user intent to update a job's restart policy."""

  def __init__(self, max_retries):
    super(JobMaxRetriesChange, self).__init__(adjusts_template=True)
    self._max_retries = max_retries

  def Adjust(self, resource):
    resource.task_template.spec.maxRetries = self._max_retries
    return resource


class JobTaskTimeoutChange(ConfigChanger):
  """Represents the user intent to update a job's instance deadline."""

  def __init__(self, timeout_seconds):
    super(JobTaskTimeoutChange, self).__init__(adjusts_template=True)
    self._timeout_seconds = timeout_seconds

  def Adjust(self, resource):
    resource.task_template.spec.timeoutSeconds = self._timeout_seconds
    return resource


class CpuThrottlingChange(ConfigChanger):
  """Sets the cpu-throttling annotation on the service template."""

  def __init__(self, throttling):
    super(CpuThrottlingChange, self).__init__(adjusts_template=True)
    self._throttling = throttling

  def Adjust(self, resource):
    resource.template.annotations[
        container_resource.CPU_THROTTLE_ANNOTATION] = str(self._throttling)
    return resource


class StartupCpuBoostChange(ConfigChanger):
  """Sets the startup-cpu-boost annotation on the service template."""

  def __init__(self, cpu_boost):
    super(StartupCpuBoostChange, self).__init__(adjusts_template=True)
    self._cpu_boost = cpu_boost

  def Adjust(self, resource):
    resource.template.annotations[
        container_resource.COLD_START_BOOST_ANNOTATION] = str(self._cpu_boost)
    return resource


class NetworkInterfacesChange(ConfigChanger):
  """Sets or updates the network interfaces annotation on the template."""

  def __init__(self, network_is_set, network, subnet_is_set, subnet,
               network_tags_is_set, network_tags):
    super(NetworkInterfacesChange, self).__init__(adjusts_template=True)
    self._network_is_set = network_is_set
    self._network = network
    self._subnet_is_set = subnet_is_set
    self._subnet = subnet
    self._network_tags_is_set = network_tags_is_set
    self._network_tags = network_tags

  def _SetOrClear(self, m, key, value):
    if value:
      # If value is present, add key=value to m.
      m[key] = value
    elif key in m:
      # Otherwise clear the key from m.
      del m[key]

  def Adjust(self, resource):
    annotations = resource.template.annotations
    network_interface = {}
    if k8s_object.NETWORK_INTERFACES_ANNOTATION in annotations:
      network_interface = json.loads(
          annotations[k8s_object.NETWORK_INTERFACES_ANNOTATION])[0]
    if self._network_is_set:
      self._SetOrClear(network_interface, 'network', self._network)
    if self._subnet_is_set:
      self._SetOrClear(network_interface, 'subnetwork', self._subnet)
    if self._network_tags_is_set:
      self._SetOrClear(network_interface, 'tags', self._network_tags)
    value = ''
    if network_interface:
      value = '[{interfaces}]'.format(
          interfaces=json.dumps(network_interface, sort_keys=True))
    self._SetOrClear(annotations, k8s_object.NETWORK_INTERFACES_ANNOTATION,
                     value)
    return resource


class CustomAudiencesChanges(ConfigChanger):
  """Represents the intent to update the custom audiences."""

  def __init__(self, args):
    """Initializes the intent to update the custom audiences.

    Args:
      args: Args to the command.
    """
    super(CustomAudiencesChanges, self).__init__(adjusts_template=True)
    self._args = args

  @property
  def add_custom_audiences(self):
    return getattr(self._args, 'add_custom_audiences', None)

  @property
  def remove_custom_audiences(self):
    return getattr(self._args, 'remove_custom_audiences', None)

  @property
  def set_custom_audiences(self):
    return getattr(self._args, 'set_custom_audiences', None)

  @property
  def clear_custom_audiences(self):
    return getattr(self._args, 'clear_custom_audiences', None)

  def Adjust(self, resource):

    def GetCurrentCustomAudiences():
      annotation_val = resource.annotations.get(
          k8s_object.CUSTOM_AUDIENCES_ANNOTATION)
      if annotation_val:
        return json.loads(annotation_val)
      return []

    audiences = repeated.ParsePrimitiveArgs(self, 'custom-audiences',
                                            GetCurrentCustomAudiences)
    if audiences is not None:
      if audiences:
        resource.annotations[
            k8s_object.CUSTOM_AUDIENCES_ANNOTATION] = json.dumps(audiences)
      elif k8s_object.CUSTOM_AUDIENCES_ANNOTATION in resource.annotations:
        del resource.annotations[k8s_object.CUSTOM_AUDIENCES_ANNOTATION]
    return resource


class RuntimeChange(ConfigChanger):
  """Sets the runtime annotation on the service template."""

  def __init__(self, runtime):
    super(RuntimeChange, self).__init__(adjusts_template=True)
    self._runtime = runtime

  def Adjust(self, resource):
    if self._runtime == 'default':
      if k8s_object.RUNTIME_ANNOTATION in resource.template.annotations:
        del resource.template.annotations[k8s_object.RUNTIME_ANNOTATION]
    else:
      resource.template.annotations[k8s_object.RUNTIME_ANNOTATION] = str(
          self._runtime
      )
    return resource
